# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-suivimp views/forms/actions/components for web ui"""
from cubicweb.selectors import is_instance
from cubicweb.web.views.tableview import EntityTableView, RelationColRenderer


###############################################################################
### GENOMIC MEASURE - CGH #####################################################
###############################################################################
class AbstractCghTableView(EntityTableView):
    __abtract__ = True
    __select__ = EntityTableView.__select__ & is_instance('CghResult')


class CghTableView(AbstractCghTableView):
    __regid__ = 'cgh-table-view'
    columns = ['related_gene', 'cgh_ratio']
    column_renderers = {'related_gene': RelationColRenderer(role='subject')}


class GeneCghTableView(AbstractCghTableView):
    __regid__ = 'gene-cgh-table-view'
    columns = ['related_measure', 'cgh_ratio']
    column_renderers = {'related_measure': RelationColRenderer(role='subject', vid='incontext')}


###############################################################################
### GENOMIC MEASURE - SEQUENCAGE ##############################################
###############################################################################
class AbstractMutationTableView(EntityTableView):
    __abstract__ = True
    __select__ = EntityTableView.__select__ & is_instance('Mutation')

    columns = ['histology_number', 'biopsy_date', 'percent_tumoral',
               'valid', 'mutation_id', 'mutation_type', 'ploidy',
               'reference_base', 'variant_base', 'variant_frequency', 'p_value',
               'coverage', 'reference_coverage', 'variant_coverage',
               'hotspot_id', 'reference_id', 'comment',
               'protein', 'biological_classification', 'classification_type',
               'locus_version', 'nucl', 'polyphen', 'sift', 'gvd_alignment',
               'base', 'conclusions', 'reception_date', 'result_date',
               'block', 'medical_coverage', 'transfert']


class MutationTableView(AbstractMutationTableView):
    __regid__ = 'mutation-table-view'

    column_renderers = {'related_gene': RelationColRenderer(role='subject')}

    columns = ['related_gene',] + AbstractMutationTableView.columns


class GeneMutationTableView(AbstractMutationTableView):
    __regid__ = 'gene-mutation-table-view'

    column_renderers = {'related_measure': RelationColRenderer(role='subject', vid='incontext')}

    columns = ['related_measure',] + AbstractMutationTableView.columns



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-brainomics views/forms/actions/components for web ui"""

from cubicweb.web.views.startup import IndexView


class BrainomicsIndexView(IndexView):

    def call(self, **kwargs):
        self.w(u'<div class="span12">')
        self.w(u'<div class="row-fluid">')
        url = self._cw.data_url('images/subject.png')
        href = self._cw.build_url(rql='Any X WHERE X is Subject')
        self.w(u'<div class="span3"><h2>Subjects</h2><a href="%s"><img width="200" src="%s"/></a></div>'
               % (href, url))
        url = self._cw.data_url('images/images.png')
        href = self._cw.build_url(rql=('Any X, XT, XL, XI, XF, XD WHERE X is Scan, '
                                       'X type XT, X label XL, X identifier XI, '
                                       'X format XF, X description XD'))
        self.w(u'<div class="span3"><h2>Images</h2><a href="%s"><img width="200" src="%s"/></a></div>'
               % (href, url))
        url = self._cw.data_url('images/genetics.png')
        href = self._cw.build_url(rql='(Any X WHERE X is GenomicMeasure) '
                                  'UNION (Any X WHERE X is GenericTestRun, X instance_of T, T type "genomics")')
        self.w(u'<div class="span3"><h2>Genetics</h2><a href="%s"><img width="200" src="%s"/></a></div>'
               % (href, url))
        url = self._cw.data_url('images/questionnaire.png')
        href = self._cw.build_url(rql='Any X, XI, XD WHERE X is QuestionnaireRun, X identifier XI, X datetime XD')
        self.w(u'<div class="span3"><h2>Questionnaires</h2><a href="%s"><img width="200" src="%s"/></a></div>'
               % (href, url))
        self.w(u'</div>')
        self.w(u'</div>')
        self.w(u'</div>')
        self.w(u'<div class="span12">')
        self.w(u'<div class="row-fluid">')
        for title in ('index', 'rqlexamples'):
            card = self._cw.execute('Any X WHERE X is Card, X title %(t)s', {'t': title})
            if card:
                self.w(card.get_entity(0,0).content)
        self.w(u'</div>')
        self.w(u'</div>')


def registration_callback(vreg):
    vreg.register_and_replace(BrainomicsIndexView, IndexView)



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.


import os
from os import path as osp
import shutil
import logging
import csv
from cubicweb import AuthenticationError
from cubicweb import cwconfig
from cubicweb.utils import make_uid
from cubicweb.server.utils import manager_userpasswd
from cubicweb.dbapi import in_memory_repo_cnx
from cubicweb.toolsutils import Command
from cubicweb.cwctl import CWCTL, check_options_consistency


def copy_file(old_fpath, new_fpath, logger=None):
    """Safe copy of a file from one path to another
    """
    if osp.isfile(old_fpath):
        if logger:
            logger.info('copy_file: Attempting to copy file %s to %s...\n'
                        % (old_fpath, new_fpath))
        try:
            shutil.copyfile(old_fpath, new_fpath)
        except IOError as ioer:
            logger.error('copy_file: %s\n' % ioer)
            raise

def update_ident_fpath(session, subject_eid, uuid, delete=False, logger=None):
    """Update the identifier and paths of a subject
    """
    if logger:
        logger.info('update_ident_fpath: processing subject %d...\n' % subject_eid)
    session.execute('SET X identifier %(i)s WHERE X eid %(e)s', {'i': uuid, 'e': subject_eid})
    rset = session.execute('(Any X WHERE X concerns S, S eid %(e)s, '
                           'X is QuestionnaireRun) UNION '
                           '(Any X WHERE S concerned_by X, X is Assessment, '
                            'S eid %(e)s)',
                           {'e': subject_eid}, build_descr=False)
    for concerned_or_assessment_eid in rset:
        concerned_or_assessment_eid = concerned_or_assessment_eid[0]
        if logger:
            logger.info('update_ident_fpath: processing related entity %d\n'
                        % concerned_or_assessment_eid)
        uuid = make_uid() + '_' + unicode(concerned_or_assessment_eid)
        session.execute('SET X identifier %(i)s WHERE X eid %(e)s',
                        {'i': uuid, 'e': concerned_or_assessment_eid})
    rset = session.execute('Any X, F WHERE X concerns S, S eid %(e)s, X filepath F, '
                           'X is IN (Scan, GenomicMeasure)',
                           {'e': subject_eid}, build_descr=False)
    for concerned_eid, concerned_fpath in rset:
        if logger:
            logger.info('update_ident_fpath: processing related entity %d\n' % concerned_eid)
        uuid = make_uid() + '_' + unicode(concerned_eid)
        session.execute('SET X identifier %(i)s WHERE X eid %(e)s', {'i': uuid, 'e': concerned_eid})
        new_filepath = uuid + '_' + osp.basename(concerned_fpath)
        full_fpath = session.entity_from_eid(concerned_eid).full_filepath
        base_fpath = full_fpath.split(concerned_fpath)[0]
        dest_fpath = osp.join(base_fpath, new_filepath)
        copy_file(full_fpath, dest_fpath, logger)
        if delete:
            try:
                os.remove(full_fpath)
            except OSError as oserr:
                if logger:
                    logger.error('update_ident_fpath: %s\n' % oserr)
        session.execute('SET X filepath %(i)s WHERE X eid %(e)s', {'i': new_filepath, 'e': concerned_eid})


def get_ids_uids_from_csv(csv_filename, logger=None):
    try:
        csvf = open(csv_filename, 'r')
    except IOError:
        if logger:
            logger.error('get_ids_uids_from_csv: External UUID CSV file not found: %s\n' % csv_filename)
        raise
    with csvf:
        dialect = csv.Sniffer().sniff(csvf.readline())
        csvf.seek(0)
        return dict(([unicode(x) for x in row] for row in csv.reader(csvf, dialect)))



class BrainomicsRemakeUidCommand(Command):
    """Change subject uids for anonymization.

    <instance>
      the identifier of the instance to anonymize
    """
    name = 'remake-uid'
    arguments = '<instance>'
    min_args = 1
    options = (
        ('log',
         {'short': 'l', 'action': 'store_true', 'default': False,
          'help': 'logging mode: logs all command operations'}),
        ('delete',
         {'short': 'd', 'action': 'store_true', 'default': False,
          'help': ('delete files from the old path, '
                   'after creating the files at the new path')}),
        ('external-uids',
         {'short': 'u', 'type': 'string',
          'help': 'specify external UID file'}),
        )

    def _init_cw_connection(self, appid):
        logger = logging.getLogger('brainomics')
        config = cwconfig.instance_configuration(appid)
        sourcescfg = config.sources()
        config.set_sources_mode(('system',))
        cnx = repo = None
        while cnx is None:
            try:
                login = sourcescfg['admin']['login']
                pwd = sourcescfg['admin']['password']
            except KeyError:
                login, pwd = manager_userpasswd()
            try:
                repo, cnx = in_memory_repo_cnx(config, login=login, password=pwd)
            except AuthenticationError:
                logger.warning('remake-uuid: wrong user/password')
            else:
                break
        session = repo._get_session(cnx.sessionid)
        return cnx, session

    def run(self, args):
        logger = logging.getLogger('brainomics')
        if len(args) > 0:
            appid = args.pop(0)
            cw_cnx, session = self._init_cw_connection(appid)
            session.set_cnxset()
            log = self.config.log
            delete = self.config.delete
            if self.config.external_uids:
                csv_filename = self.config.external_uids
                ids_uids = get_ids_uids_from_csv(csv_filename, logger)
            if log:
                log_file = logging.FileHandler(appid + '.log')
                logger.addHandler(log_file)
            rset = session.execute('Any X, I WHERE X is Subject, X identifier I', build_descr=False)
            for subject_eid, subject_identifier in rset:
                if not self.config.external_uids:
                    uuid = unicode(make_uid())
                else:
                    try:
                        uuid = ids_uids[subject_identifier]
                    except KeyError:
                        if logger:
                            logger.error('remake-uid: New identifier not found in %s '
                                         'for identifier %s; keeping the old one \n'
                                         % (csv_filename, subject_identifier))
                        session.rollback()
                        raise
                update_ident_fpath(session, subject_eid, uuid, delete, logger)
            session.commit()
        else:
            logger.error("remake-uuid: Please specify Brainomics instance")


CWCTL.register(BrainomicsRemakeUidCommand)



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-suivimp views/forms/actions/components for web ui"""
from logilab.mtconverter import xml_escape
from cubicweb.selectors import is_instance
from cubicweb.web.views.primary import PrimaryView

from cubes.brainomics.entities import MEASURES


###############################################################################
### ABSTRACT PRIMARY VIEW #####################################################
###############################################################################
class BrainomicsPrimaryView(PrimaryView):
    __select__ = PrimaryView.__select__
    __abstract__ = True

    def iterate_attributes(self, entity):
        return []

    def iterate_data(self, entity):
        return []

    def _build_data(self, data, rql, label):
        rset = self._cw.execute(rql)
        if len(rset):
            url = self._cw.build_url(rql=rql)
            ahref = u'<a href="%s">%s</a>' % (url, self._cw._('See all'))
            data.append((label, (len(rset), ahref)))
        return data

    def display_main_col(self, entity):
        pass

    def display_additional_header(self, entity):
        pass

    def display_header(self, entity):
        w = self.w
        w(u'<div class="well">')
        w(u'<div class="page-header">')
        w(u'<h2>%s</h2>' % xml_escape(entity.dc_title()))
        w(u'</div>')
        w(u'<dl class="dl-horizontal">')
        for label, attribute in self.iterate_attributes(entity):
            if attribute:
                w(u'<dt>%s</dt><dd>%s</dd>' % (self._cw._(label), attribute))
        w(u'</dl>')
        # Additional header
        self.display_additional_header(entity)
        # Data
        self.display_data_table(entity)
        w(u'</div>')

    def display_data_table(self, entity):
        w = self.w
        table_data = self.iterate_data(entity)
        if not table_data:
            return
        w(u'<h3>%s</h3>' % self._cw._('Data overview'))
        w(u'<table class="table table-striped table-bordered table-condensed">')
        for label, data in table_data:
            w(u'<tr>')
            w(u'<th>%s</th>' % self._cw._(label))
            for d in data:
                w(u'<th>%s</th>' % d)
            w(u'</tr>')
        w(u'</table>')

    def call(self, rset=None):
        entity = self.cw_rset.get_entity(0,0)
        w = self.w
        # Tabblables
        w(u'<div class="tab-content">')
        # Mainn table
        w(u'<div class="tab-pane active" id="entity-mainview">')
        # Header
        self.display_header(entity)
        # Main col
        w(u'<div class="row-fluid">')
        self.display_main_col(entity)
        w(u'</div>')
        # Close table
        w(u'</div>')
        # Add other tables
        if 'ctx-entity-tabs' in self._cw.vreg:
            tabs = self._cw.vreg['ctx-entity-tabs'].possible_objects(self._cw, rset=self.cw_rset)
            for tab in tabs:
                tab.render(self.w)
        w(u'</div>')


###############################################################################
### SUBJECT ###################################################################
###############################################################################
class SubjectPrimaryView(BrainomicsPrimaryView):
    __select__ = BrainomicsPrimaryView.__select__ & is_instance('Subject')

    def iterate_attributes(self, entity):
        return [(self._cw._('Age'), entity.display_age_for_assessments()),
                (self._cw._('Handedness'), entity.handedness),
                (self._cw._('Gender'), entity.gender),]

    def display_additional_header(self, entity):
        scores = self._cw.execute('Any X WHERE S related_infos X, S eid %(e)s', {'e': entity.eid})
        if scores:
            self.w(u'<h3>%s</h3>' % self._cw._('Scores'))
            self.wview('list', scores)

    def iterate_data(self, entity):
        data = []
        for measure in MEASURES:
            rql = 'Any X WHERE X is %s, X concerns S, S eid %s' % (measure, entity.eid)
            data = self._build_data(data, rql, self._cw._(measure))
        return data

    def display_main_col(self, entity):
        self.w(u'<div class="span6">')
        # Detailled assessments
        rset = self._cw.execute('Any X WHERE S concerned_by X, S eid %(e)s', {'e': entity.eid})
        if rset:
            self.w(u'<h3>%s</h3>' % self._cw._('Detailed Assessments'))
            self.wview('list', rset=rset)
        self.w(u'</div>')
        self.w(u'<div class="span5">')
        # Related external resources
        rset = self._cw.execute('Any E WHERE X external_resources E, S concerned_by X, S eid %(e)s, '
                                'NOT EXISTS(X generates M)', {'e': entity.eid})
        if rset:
            self.w(u'<h3>%s</h3>' % self._cw._('External Resources'))
            self.wview('list', rset=rset)
        self.w(u'</div>')


############################################################################
### STUDY ##################################################################
############################################################################
class StudyPrimaryView(BrainomicsPrimaryView):
    __select__ = BrainomicsPrimaryView.__select__ & is_instance('Study')

    def iterate_attributes(self, entity):
        return [(self._cw._('Name'), entity.name),
                (self._cw._('Description'), entity.description),
                (self._cw._('Keywords'), entity.keywords),
                (self._cw._('Themes'), ', '.join([t.dc_title() for t in entity.themes]))]

    def iterate_data(self, entity):
        data = []
        # Subjects
        rql = 'Any X WHERE X related_studies S, S eid %s' % entity.eid
        data = self._build_data(data, rql, self._cw._('Subjects'))
        # Measures
        rset = self._cw.execute('Any M, COUNT(X) GROUPBY S, M WHERE X related_study S, '
                                'S eid %s, X is E, E name M' % entity.eid)
        for label, count in rset:
            rql='Any X WHERE X is %s, X related_study S, S eid %s' % (label, entity.eid)
            data = self._build_data(data, rql, self._cw._(label))
        return data


###############################################################################
### INVESTIGATOR ##############################################################
###############################################################################
class InvestigatorPrimaryView(BrainomicsPrimaryView):
    __select__ = BrainomicsPrimaryView.__select__ & is_instance('Investigator')

    def iterate_attributes(self, entity):
        return [(self._cw._('Firstname'), entity.firstname),
                (self._cw._('Lastname'), entity.lastname),
                (self._cw._('Title'), entity.title),
                (self._cw._('Institution'), entity.institution),
                (self._cw._('Department'), entity.department),]

    def display_main_col(self, entity):
        # Detailled assessments
        # XXX Direct link to the measures/runs ?
        self.w(u'<h3>%s</h3>' % self._cw._('Assessments conducted by this investigator'))
        rset = self._cw.execute('Any A WHERE A conducted_by X, X eid %(e)s', {'e': entity.eid})
        if rset:
            self.wview('list', rset)


############################################################################
### CENTER #################################################################
############################################################################
class CenterPrimaryView(BrainomicsPrimaryView):
    __select__ = BrainomicsPrimaryView.__select__ & is_instance('Center')

    def iterate_attributes(self, entity):
        return [(self._cw._('Identifier'), entity.identifier),
                (self._cw._('Department'), entity.department),
                (self._cw._('City'), entity.city),
                (self._cw._('Country'), entity.country),]

    def iterate_data(self, entity):
        data = []
        # Subjects
        rql = 'DISTINCT Any S WHERE S is Subject, X holds A, S concerned_by A, X eid %s' % entity.eid
        data = self._build_data(data, rql, self._cw._('Subjects'))
        # Assessments
        rql = 'DISTINCT Any A WHERE X holds A, X eid %s' % entity.eid
        data = self._build_data(data, rql, self._cw._('Assessments'))
        return data

    def display_main_col(self, entity):
        # Devices
        rset = self._cw.execute('Any S WHERE S is Device, '
                                'S hosted_by X, X eid %(e)s', {'e': entity.eid})
        if rset:
            self.w(u'<h3>Devices</h3>')
            self.wview('list', rset=rset)


############################################################################
### DEVICE #################################################################
############################################################################
class DevicePrimaryView(BrainomicsPrimaryView):
    __select__ = BrainomicsPrimaryView.__select__ & is_instance('Device')

    def iterate_attributes(self, entity):
        center = entity.hosted_by[0]
        center = u'<a href="%s">%s</a>' % (center.absolute_url(), center.dc_title())
        return [(self._cw._('Name'), entity.name),
                (self._cw._('Manufacturer'), entity.manufacturer),
                (self._cw._('Model'), entity.model),
                (self._cw._('Hosted by'), center),]

    def iterate_data(self, entity):
        data = []
        rql = 'DISTINCT Any S WHERE S uses_device X, X eid %s' % entity.eid
        data = self._build_data(data, rql, self._cw._('Generated measures'))
        return data


############################################################################
### ASSESSMENT #############################################################
############################################################################
class AssessmentPrimaryView(BrainomicsPrimaryView):
    __select__ = BrainomicsPrimaryView.__select__ & is_instance('Assessment')

    def iterate_attributes(self, entity):
        subject = entity.reverse_concerned_by[0]
        subject = u'<a href="%s">%s</a>' % (subject.absolute_url(), subject.dc_title())
        return [(self._cw._('Identifier'), entity.identifier),
                (self._cw._('Protocol'), entity.protocol),
                (self._cw._('Date'), entity.datetime),
                (self._cw._('Subject'), subject),
                (self._cw._('Study'), entity.related_study[0].view('outofcontext')),
                (self._cw._('Center'), entity.reverse_holds[0].view('outofcontext')),
                ]

    def iterate_data(self, entity):
        data = []
        # Generated
        rset = self._cw.execute('Any M, COUNT(S) GROUPBY M WHERE X generates S, X eid %s, '
                                'S is E, E name M' % entity.eid)
        for m, count in rset:
            rql = 'Any S WHERE X generates S, S is %s, X eid %s' % (m, entity.eid)
            data = self._build_data(data, rql, self._cw._('Generated %(s)s' % {'s': m}))
        # Used
        rset = self._cw.execute('Any M, COUNT(S) GROUPBY M WHERE X uses S, X eid %s, '
                                'S is E, E name M' % entity.eid)
        for m, count in rset:
            rql = 'Any S WHERE X uses S, S is %s, X eid %s' % (m, entity.eid)
            data = self._build_data(data, rql, self._cw._('Used %(s)s' % {'s': m}))
        return data


###############################################################################
### SCOREDEF ##################################################################
###############################################################################
class ScoreDefinitionPrimaryView(BrainomicsPrimaryView):
    __select__ = BrainomicsPrimaryView.__select__ & is_instance('ScoreDefinition')

    def iterate_attributes(self, entity):
        return [(self._cw._('Name'), entity.name),
                (self._cw._('Category'), entity.category),
                (self._cw._('Type'), entity.type),
                (self._cw._('Unit'), entity.unit),
                (self._cw._('Possible values'), entity.possible_values)
                ]

    def display_main_col(self, entity):
        # Scores
        w = self.w
        rset = self._cw.execute('Any S,V,T,DA WHERE S is ScoreValue, S value V, S text T, '
                                'S datetime DA, S definition D, D eid %(e)s', {'e': entity.eid})
        if not rset:
            return
        w(u'<table class="table table-striped table-bordered table-condensed">')
        w(u'<tr><th>%s</th><th>%s</th><th>%s</th></tr>'
          % (self._cw._('Subject'), self._cw._('value'), self._cw._('datetime')))
        # Warm subjects
        score = rset.get_entity(0, 0)
        if score.reverse_related_infos:
            subjects_rset = self._cw.execute('Any SB, S, I WHERE S is ScoreValue, '
                                             'SB related_infos S, SB identifier I, '
                                             'S definition D, D eid %(e)s', {'e': entity.eid})
        else:
            # Measure/concerns
            subjects_rset = self._cw.execute('Any SB, S, I WHERE S is ScoreValue, '
                                             'S measure M, M concerns SB, SB identifier I, '
                                             'S definition D, D eid %(e)s', {'e': entity.eid})
        subjects = {}
        for ind, e in enumerate(subjects_rset.entities()):
            subjects[subjects_rset[ind][1]] = e
        # Plot results
        for ind in range(len(rset)):
            score = rset.get_entity(ind, 0)
            subj = subjects[score.eid]
            if subj:
                subject = u'<a href="%s">%s</a>' % (subj.absolute_url(), subj.identifier)
            else:
                subject = u'<unknown - error>'
            w(u'<tr><th>%s</th><th>%s</th><th>%s</th></tr>'
              % (subject, score.complete_value, score.datetime or '-'))
        w(u'</table>')


###############################################################################
### GENERIC TEST RUN ##########################################################
###############################################################################
class GenericTestRunPrimaryView(BrainomicsPrimaryView):
    __select__ = BrainomicsPrimaryView.__select__ & is_instance('GenericTestRun')

    def iterate_attributes(self, entity):
        test = entity.instance_of[0]
        test = u'<a href="%s">%s</a>' % (test.absolute_url(), test.dc_title())
        subject = entity.concerns[0]
        subject = u'<a href="%s">%s</a>' % (subject.absolute_url(), subject.dc_title())
        return [(self._cw._('Date'), entity.datetime),
                (self._cw._('Instance of'), test),
                (self._cw._('Subject'), subject),]

    def display_main_col(self, entity):
        w = self.w
        # Scores
        self.w(u'<div class="span9">')
        rset = self._cw.execute('Any SD, S WHERE QR is GenericTestRun, QR eid %(e)s, '
                                'S measure QR, S definition SD, S value V, S text T',
                                {'e': entity.eid})
        w(u'<table class="table table-striped table-bordered table-condensed">')
        w(u'<tr>')
        for label in ('score', 'text/value'):
                        w(u'<th>%s</th>' % self._cw._(label))
        w(u'</tr>')
        for ind, scoredef in enumerate(rset.entities()):
            w(u'<tr>')
            w(u'<th><a href="%s">%s</a></th>' % (scoredef.absolute_url(), scoredef.dc_title()))
            w(u'<th>%s</th>' % (rset.get_entity(ind, 1).complete_value))
            w(u'</tr>')
        w(u'</table>')
        self.w(u'</div>')
        # Related external resources
        self.w(u'<div class="span3">')
        rset = self._cw.execute('(Any E WHERE X external_resources E, X eid %(e)s) '
                                'UNION (Any E WHERE A generates X, A external_resources E, X eid %(e)s)',
                                {'e': entity.eid})
        if rset:
            w(u'<h3>%s</h3>' % self._cw._('External Resources'))
            self.wview('list', rset=rset)
        self.w(u'</div>')


###############################################################################
### GENERIC TEST ##############################################################
###############################################################################
class GenericTestPrimaryView(BrainomicsPrimaryView):
    __select__ = BrainomicsPrimaryView.__select__ & is_instance('GenericTest')

    def iterate_attributes(self, entity):
        return [(self._cw._('Identifier'), entity.identifier),
                (self._cw._('Name'), entity.name),
                (self._cw._('Type'), entity.type),
                (self._cw._('Version'), entity.version),
                (self._cw._('Language'), entity.language),
                (self._cw._('Note'), entity.note),]

    def iterate_data(self, entity):
        data = []
        rql = 'Any S WHERE S instance_of X, X eid %s' % entity.eid
        data = self._build_data(data, rql, self._cw._('Test runs'))
        return data


###############################################################################
### EXTERNAL RESOURCE #########################################################
###############################################################################
class ExternalResourcePrimaryView(BrainomicsPrimaryView):
    __select__ = BrainomicsPrimaryView.__select__ & is_instance('ExternalResource')

    def iterate_attributes(self, entity):
        study = entity.related_study
        study = u'<a href="%s">%s</a>' % (study[0].absolute_url(), study[0].dc_title()) if study else u''
        return [(self._cw._('Name'), entity.name),
                (self._cw._('Filepath'), entity.filepath),
                (self._cw._('Study'), study),
                ]


###############################################################################
### QUESTIONNAIRERUN ##########################################################
###############################################################################
class QuestionnaireRunPrimaryView(BrainomicsPrimaryView):
    __select__ = BrainomicsPrimaryView.__select__ & is_instance('QuestionnaireRun')

    def iterate_attributes(self, entity):
        questionnaire = entity.instance_of[0]
        questionnaire = u'<a href="%s">%s</a>' % (questionnaire.absolute_url(), questionnaire.dc_title())
        subject = entity.concerns[0]
        subject = u'<a href="%s">%s</a>' % (subject.absolute_url(), subject.dc_title())
        assessment = entity.reverse_generates[0]
        assessment = u'<a href="%s">%s</a>' % (assessment.absolute_url(), assessment.dc_title())
        return [(self._cw._('Date'), entity.datetime),
                (self._cw._('Instance of'), questionnaire),
                (self._cw._('Subject'), subject),
                (self._cw._('Assessment'), assessment),
                ]


    def display_additional_header(self, entity):
        w = self.w
        # Scores and External resources
        rset = self._cw.execute('Any X WHERE X is ScoreValue, X measure Q, Q eid %(q)s',
                                {'q': entity.eid})
        if rset:
            w(u'<h3>%s</h3>' % self._cw._('Additional scores'))
            self.wview('list', rset=rset)
        rset = self._cw.execute('Any X WHERE X is ExternalResource, Q external_resources X, Q eid %(q)s',
                                {'q': entity.eid})
        if rset:
            w(u'<h3>%s</h3>' % self._cw._('Additional resources'))
            self.wview('list', rset=rset)

    def display_main_col(self, entity):
        w = self.w
        # Results file
        w(entity.view('results-view'))
        # Answers
        w(u'<h3>%s</h3>' % self._cw._('Answers'))
        rset = self._cw.execute('Any Q, A, AV, AD ORDERBY QP '
                                'WHERE QR is QuestionnaireRun, QR eid %(e)s, '
                                'A questionnaire_run QR, A question Q, '
                                'A value AV, A datetime AD, '
                                'Q identifier QI, Q position QP, Q text QT, '
                                'Q type QTY, Q possible_answers QPA',
                                {'e': entity.eid})
        w(u'<table class="table table-striped table-bordered table-condensed">')
        w(u'<tr>')
        for label in ('question', 'text', 'value', 'datetime'):
            w(u'<th>%s</th>' % self._cw._(label))
        w(u'</tr>')
        for ind, question in enumerate(rset.entities()):
            w(u'<tr>')
            w(u'<th><a href="%s">%s</a></th>' % (question.absolute_url(), question.identifier))
            w(u'<th>%s</th>' % question.text)
            answer = rset.get_entity(ind, 1)
            w(u'<th>%s</th>' % answer.computed_value)
            w(u'<th>%s</th>' % (answer.datetime or self._cw._(u'unknown')))
            w(u'</tr>')
        w(u'</table>')


###############################################################################
### QUESTIONNAIRE #############################################################
###############################################################################
class QuestionnairePrimaryView(BrainomicsPrimaryView):
    __select__ = BrainomicsPrimaryView.__select__ & is_instance('Questionnaire')

    def iterate_attributes(self, entity):
        return [(self._cw._('Name'), entity.name),
                (self._cw._('Type'), entity.type),
                (self._cw._('Version'), entity.version),
                (self._cw._('Language'), entity.language),
                (self._cw._('Note'), entity.note),
                ]

    def iterate_data(self, entity):
        data = []
        rql = 'Any S WHERE S is QuestionnaireRun, S instance_of X, X eid %s' % entity.eid
        data = self._build_data(data, rql, self._cw._('Related runs'))
        return data

    def display_main_col(self, entity):
        # Results file
        self.w(entity.view('results-view'))
        # Questions
        w = self.w
        rset = self._cw.execute('Any Q, QI, QP, QT, QTY, QPA ORDERBY QP '
                                'WHERE QR is Questionnaire, QR eid %(e)s, '
                                'Q questionnaire QR, '
                                'Q identifier QI, Q position QP, Q text QT, '
                                'Q type QTY, Q possible_answers QPA',
                                {'e': entity.eid})
        w(u'<table class="table table-striped table-bordered table-condensed">')
        w(u'<tr>')
        for label in ('question', 'text', 'type', 'possible_answers', 'Answers'):
            w(u'<th>%s</th>' % self._cw._(label))
        w(u'</tr>')
        for question in rset.entities():
            w(u'<tr>')
            possible_answers = question.displayable_possible_answers
            for value in (question.identifier, question.text, question.type, possible_answers):
                w(u'<th>%s</th>' % value)
            w(u'<th><a href="%s">%s</a></th>' % (question.absolute_url(), self._cw._('See detailled question')))
            w(u'</tr>')
        w(u'</table>')


###############################################################################
### QUESTION ##################################################################
###############################################################################
class QuestionPrimaryView(BrainomicsPrimaryView):
    __select__ = BrainomicsPrimaryView.__select__ & is_instance('Question')

    def iterate_attributes(self, entity):
        questionnaire = entity.questionnaire[0]
        questionnaire = u'<a href="%s">%s</a>' % (questionnaire.absolute_url(), questionnaire.dc_title())
        return [(self._cw._('Questionnaire'), questionnaire),
                (self._cw._('Position'), entity.position),
                (self._cw._('Text'), entity.text),
                (self._cw._('Type'), entity.type),
                (self._cw._('Possible answers'), entity.possible_answers),
                ]

    def display_main_col(self, entity):
        # Answers
        w = self.w
        rset = self._cw.execute('Any A, AQ, S, Q, AV, AD, AI, I, QT, QPA WHERE A is Answer, '
                                'A question Q, Q eid %(e)s, Q type QT, Q possible_answers QPA, '
                                'A value AV, A datetime AD, AQ identifier AI, '
                                'A questionnaire_run AQ, AQ concerns S, S identifier I',
                                {'e': entity.eid})
        w(u'<table class="table table-striped table-bordered table-condensed">')
        w(u'<tr>')
        for label in ('answer',  'value', 'datetime'):
            w(u'<th>%s</th>' % self._cw._(label))
        w(u'</tr>')
        for ind, answer in enumerate(rset.entities()):
            # XXX WARM CACHE ?
            qrun = rset.get_entity(ind, 1)
            subject = rset.get_entity(ind, 2)
            question = rset.get_entity(ind, 3)
            w(u'<tr>')
            w(u'<th><a href="%s">%s</a></th>' % (answer.absolute_url(), answer.dc_title()))
            w(u'<th>%s</th>' % answer.computed_value)
            w(u'<th>%s</th>' % (answer.datetime or self._cw._(u'unknown')))
            w(u'</tr>')
        w(u'</table>')


###############################################################################
### SCAN ######################################################################
###############################################################################
class ScanPrimaryView(BrainomicsPrimaryView):
    __select__ = BrainomicsPrimaryView.__select__ & is_instance('Scan')

    def iterate_attributes(self, entity):
        subject = entity.concerns[0]
        subject = u'<a href="%s">%s</a>' % (subject.absolute_url(), subject.dc_title())
        device = entity.uses_device
        if entity.uses_device:
            device = u'<a href="%s">%s</a>' % (device[0].absolute_url(), device[0].dc_title())
        else:
            device = None
        assessment = entity.reverse_generates[0]
        assessment = u'<a href="%s">%s</a>' % (assessment.absolute_url(), assessment.dc_title())
        return [(self._cw._('Identifier'), entity.identifier),
                (self._cw._('Label'), entity.label),
                (self._cw._('Type'), entity.type),
                (self._cw._('Format'), entity.format),
                (self._cw._('Position in acquisition'), entity.position_acquisition),
                (self._cw._('Subject'), subject),
                (self._cw._('Device'), device),
                (self._cw._('Assessment'), assessment),
                ]

    def display_header(self, entity):
        w = self.w
        w(u'<div class="well">')
        w(u'<div class="page-header">')
        w(u'<h2>%s</h2>' % xml_escape(entity.dc_title()))
        w(u'</div>')
        w(u'<dl class="dl-horizontal">')
        for label, attribute in self.iterate_attributes(entity):
            if attribute:
                w(u'<dt>%s</dt><dd>%s</dd>' % (self._cw._(label), attribute))
        # Add data info
        data = entity.has_data
        if data:
            data[0].view('scan-data-view', w=w)
        w(u'</dl>')
        # Additional header
        self.display_additional_header(entity)
        # Data
        self.display_data_table(entity)
        w(u'</div>')

    def display_main_col(self, entity):
        # Results file
        self.w(entity.view('results-view'))
        # Related external resources
        rset = self._cw.execute('(Any E WHERE X external_resources E, X eid %(e)s) '
                                'UNION (Any E WHERE A generates X, A external_resources E, X eid %(e)s)',
                                {'e': entity.eid})
        if rset:
            self.w(u'<h3>%s</h3>' % self._cw._('External Resources'))
            self.wview('list', rset=rset)


###############################################################################
### GENOMIC MEASURE ###########################################################
###############################################################################
class GenomicMeasurePrimaryView(BrainomicsPrimaryView):
    __select__ = BrainomicsPrimaryView.__select__ & is_instance('GenomicMeasure')

    def iterate_attributes(self, entity):
        subject = entity.concerns[0]
        subject = u'<a href="%s">%s</a>' % (subject.absolute_url(), subject.dc_title())
        fields = [(self._cw._('Type'), entity.type),
                  (self._cw._('Format'), entity.format),
                  (self._cw._('File path'), entity.filepath),
                  (self._cw._('Chip S/N'), entity.formatted_chip_serialnum),
                  (self._cw._('Subject'), subject),
                  (self._cw._('Study'), entity.related_study[0].view('outofcontext'))]
        if entity.platform:
            fields.append((self._cw._('Platform'), entity.platform[0].view('outofcontext')))
        return fields

    def display_main_col(self, entity):
        self.w(entity.view('results-view'))
        cgh_results = self._cw.execute('Any R WHERE R is CghResult, '
                                       'R related_measure X, X eid %(e)s',
                                       {'e': entity.eid})
        mutation_results = self._cw.execute('Any R WHERE R is Mutation, '
                                            'R related_measure X, X eid %(e)s',
                                            {'e': entity.eid})
        if cgh_results or mutation_results:
            self.w(u'<div class="span3">')
            if cgh_results:
                self.w(u'<h3>%s</h3>' % self._cw._('CGH results'))
                self.wview('cgh-table-view', rset=cgh_results)
            if mutation_results:
                self.w(u'<h3>%s</h3>' % self._cw._('Sequencing results'))
                self.wview('mutation-table-view', rset=mutation_results)
            self.w(u'</div>')


###############################################################################
### GENE ######################################################################
###############################################################################
class GenePrimaryView(BrainomicsPrimaryView):
    __select__ = BrainomicsPrimaryView.__select__ & is_instance('Gene')

    def iterate_attributes(self, entity):
        fields = [(self._cw._('Identifier'), entity.gene_id),
                  (self._cw._('Start position'), entity.start_position),
                  (self._cw._('Stop position'), entity.stop_position),
                  (self._cw._('Chromosome'), entity.chromosome[0].view('outofcontext'))]
        return fields

    def display_main_col(self, entity):
        cgh_results = self._cw.execute('Any R WHERE R is CghResult, '
                                       'R related_gene X, X eid %(e)s',
                                       {'e': entity.eid})
        mutation_results = self._cw.execute('Any R WHERE R is Mutation, '
                                            'R related_gene X, X eid %(e)s',
                                            {'e': entity.eid})
        if cgh_results or mutation_results:
            self.w(u'<div class="span3">')
            if cgh_results:
                self.w(u'<h3>%s</h3>' % self._cw._('CGH results'))
                self.wview('gene-cgh-table-view', rset=cgh_results)
            if mutation_results:
                self.w(u'<h3>%s</h3>' % self._cw._('Sequencing results'))
                self.wview('gene-mutation-table-view', rset=mutation_results)
            self.w(u'</div>')



# -*- coding: utf-8 -*-
# copyright 2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

import sys
import os.path as osp
import glob

import nibabel as nb

from cubicweb.dataimport import SQLGenObjectStore as Store

from cubes.brainomics.importers.helpers import get_image_info


if __name__ == '__main__':
    base_path = osp.abspath(sys.argv[4])
    store = Store(session)
    count_all_subject = 0
    global_study = store.create_entity('Study', data_filepath=unicode(base_path), name = u'openfmri',
                                       description=u'''OpenfMRI.org is a project dedicated to the free and open sharing of functional magnetic resonance imaging (fMRI) datasets, including raw data. See http://openfmri.org/''')
    # All studies
    for study_path in glob.iglob(osp.join(base_path,'*/')):
        study_id = study_path[:-1].split('/')[-1]
        print '--->', study_id
        # XXX use True metadata for studies, P.I. ...
        study = store.create_entity('Study', data_filepath=unicode(osp.abspath(study_path)),
                                    name = u'openfmri-%s' % study_id, keywords='openfmri')
        # Device
        device_path = osp.join(study_path, 'scan_key.txt')
        device_eid = None
        if osp.exists(device_path):
            device = open(device_path).readline()
            if device.strip():
                device = store.create_entity('Device', name=unicode(device.strip()),
                                             model=unicode(device.strip()))
                device_eid = device.eid
        # Tasks definition
        task_path = osp.join(study_path, 'task_key.txt')
        task_eid = None
        if osp.exists(task_path):
            ext_resource = store.create_entity('ExternalResource', name=u'task_key',
                                               related_study=study.eid,
                                               filepath=unicode(osp.relpath(task_path, start=study_path)))
            task_eid = ext_resource.eid
        # Model contrasts and condition_key
        model_path = osp.join(study_path, 'models')
        model_files = {}
        if osp.exists(model_path):
            for model in glob.iglob(osp.join(model_path, 'model*/*.txt')):
                model_id = model.split('/')[-2]
                file_id = model.split('/')[-1]
                ext_resource = store.create_entity('ExternalResource',
                                                   name=unicode(osp.splitext(file_id)[0]),
                                                   related_study=study.eid,
                                                   filepath=unicode(osp.relpath(model, start=study_path)))
                model_files.setdefault(model_id, []).append(ext_resource.eid)
        # Demographics
        demographics_path = osp.join(study_path, 'demographics.txt')
        demographics = {}
        if osp.exists(demographics_path):
            for line in open(demographics_path):
                infos = line.strip().split('\t')
                if len(infos) != 4:
                    print '!!!!', infos
                demographics[infos[1]] = {'gender': infos[2], 'age': int(infos[3])}
        # Subjects
        count_subject = 0
        for subject_path in glob.iglob(osp.join(study_path,'sub*')):
            subject_id = study_id + '_' + subject_path.split('/')[-1]
            print '--->', subject_id
            age, gender = None, None
            if subject_id in demographics:
                age = demographics[subject_id].get('age')
                gender = demographics[subject_id].get('gender')
            subject = store.create_entity('Subject', identifier=unicode(subject_id),
                                          age = age, gender=gender or u'unknown', handedness=u'unknown')
            store.relate(subject.eid, 'related_studies', study.eid)
            store.relate(subject.eid, 'related_studies', global_study.eid)
            # anatomy
            assessment = store.create_entity('Assessment',identifier=u'anat', protocol=u'anat',
                                             related_study=study.eid)
            store.relate(subject.eid, 'concerned_by', assessment.eid)
            for image in glob.iglob(osp.join(study_path, subject_path, 'anatomy/*.nii.gz')):
                image_id = image.split('/')[-1].split('.')[0]
                anat = {'identifier': u'%s_%s' % (subject_id, image_id),
                        'label': u'anatomy',
                        'format': u'nii.gz',
                        'type': u'normalized T1',
                        'related_study': study.eid,
                        'filepath': unicode(osp.relpath(image, start=study_path)),
                        'valid': True, 'completed': True, 'description': 'Image from OpenFmri'}
                mri_data = store.create_entity('MRIData', sequence=u'T1')
                # MRI data
                mri_data = {'sequence': u'T1'}
                mri_data.update(get_image_info(image))
                mri_data = store.create_entity('MRIData', **mri_data)
                anat['has_data'] = mri_data.eid
                anat = store.create_entity('Scan', **anat)
                if device_eid:
                    store.relate(anat.eid, 'uses_device', device_eid)
                store.relate(anat.eid, 'concerns', subject.eid, subjtype='Scan')
                store.relate(assessment.eid, 'generates', anat.eid, subjtype='Assessment')
            # Model
            models = {}
            for model in glob.iglob(osp.join(study_path, subject_path, 'model/*')):
                for model_path in glob.iglob(osp.join(model, '*/*/*.txt')):
                    model_id = unicode(osp.splitext(model_path.split('/')[-1])[0])
                    task_id = unicode(model_path.split('/')[-2])
                    type_id = unicode(model_path.split('/')[-3])
                    name = u'_'.join((subject_id, type_id, task_id))
                    ext_resource = store.create_entity('ExternalResource',
                                                       name=name,
                                                       related_study=study.eid,
                                                       filepath=unicode(osp.relpath(model_path, start=study_path)))
                    models.setdefault(task_id, []).append(ext_resource.eid)
            # Behav
            behavs = {}
            for behav_path in glob.iglob(osp.join(study_path, subject_path, 'behav/*')):
                    behav_id = unicode(osp.splitext(behav_path.split('/')[-1])[0])
                    task_id = unicode(behav_path.split('/')[-2])
                    type_id = unicode(behav_path.split('/')[-3])
                    name = u'_'.join((subject_id, type_id, task_id))
                    ext_resource = store.create_entity('ExternalResource',
                                                       name=name,
                                                       related_study=study.eid,
                                                       filepath=unicode(osp.relpath(behav_path, start=study_path)))
                    behavs.setdefault(task_id, []).append(ext_resource.eid)
            # Bold
            for run in glob.iglob(osp.join(study_path, subject_path, 'BOLD/*')):
                run_id = run.split('/')[-1].split('.')[0]
                assessment = store.create_entity('Assessment',identifier=unicode(run), protocol=u'fmri',
                                             related_study=study.eid)
                store.relate(subject.eid, 'concerned_by', assessment.eid)
                if task_eid:
                    store.relate(assessment.eid, 'external_resources', task_eid)
                image_id = unicode('bold')
                anat = {'identifier': u'%s_%s' % (subject_id, image_id),
                        'label': u'bold',
                        'format': u'nii.gz',
                        'type': u'raw fMRI',
                        'related_study': study.eid,
                        'filepath': unicode(osp.relpath(run+'/bold.nii.gz', start=study_path)),
                        'valid': True, 'completed': True, 'description': 'Image from OpenFmri'}
                mri_data = store.create_entity('MRIData', sequence=u'T1')
                # MRI data
                mri_data = {'sequence': u'EPI'}
                mri_data.update(get_image_info(image))
                mri_data = store.create_entity('MRIData', **mri_data)
                anat['has_data'] = mri_data.eid
                anat = store.create_entity('Scan', **anat)
                store.relate(anat.eid, 'concerns', subject.eid, subjtype='Scan')
                store.relate(assessment.eid, 'generates', anat.eid, subjtype='Assessment')
                if device_eid:
                    store.relate(anat.eid, 'uses_device', device_eid)
                for eid in models.get(run_id, []):
                    store.relate(anat.eid, 'external_resources', eid)
                for eid in behavs.get(run_id, []):
                    store.relate(anat.eid, 'external_resources', eid)
            count_subject += 1
        count_all_subject += count_subject
        print '-------> %s subjects imported for study %s' % (count_subject, study_id)
        # Flush
        store.flush()
    store.commit()
    print '-------> %s subjects imported for all_studies' % count_all_subject



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

from cubicweb.web import facet
from cubicweb.selectors import is_instance


############################################################################
### MEASURE FACETS #########################################################
############################################################################
class MeasureAgeFacet(facet.RQLPathFacet):
    __regid__ = 'measure-age-facet'
    __select__ = is_instance('Scan', 'QuestionnaireRun', 'GenomicMeasure')
    path = ['S generates X', 'S age_for_assessment A']
    order = 1
    filter_variable = 'A'
    title = _("Subject Age")

class MeasureHandednessFacet(facet.RQLPathFacet):
    __regid__ = 'measure-handedness-facet'
    __select__ = is_instance('Scan', 'QuestionnaireRun', 'GenomicMeasure')
    path = ['X concerns S', 'S handedness H']
    order = 2
    filter_variable = 'H'
    title = _('Subject Handedness')

class MeasureGenderFacet(facet.RQLPathFacet):
    __regid__ = 'measure-gender-facet'
    __select__ = is_instance('Scan', 'QuestionnaireRun', 'GenomicMeasure')
    path = ['X concerns S', 'S gender G']
    order = 3
    filter_variable = 'G'
    title = _('Subject Gender')



"""cubicweb-brainomics application package

Cube for the Brainomics Project, see http://www.brainomics.net/
"""

ALL_MEASURES = ('GenericMeasure', 'QuestionnaireRun', 'GenericTestRun', 'Scan', 'GenomicMeasure')
COMMENTED_ENTITIES = ('Study', 'Subject', 'Center', 'Device', 'Scan', 'GenomicMeasure',
                      'QuestionnaireRun', 'Questionnaire', 'Question')
WIKI_ENTITIES = ('Question', 'Questionnaire', 'Study', 'ScoreDefinition')



# -*- coding: utf-8 -*-

import sys
import os
import os.path as osp
import re
import glob
import csv
import json
import pickle
from datetime import datetime

import nibabel as nb

from cubes.brainomics.importers.helpers import (get_image_info, import_genes,
                                                import_chromosomes, import_snps)


###############################################################################
### Global Mappings ###########################################################
###############################################################################
GENDER_MAP = {
    '1': u'male',
    '2': u'female',
    '3': u'unknown',
}

HANDEDNESS_MAP = {
    'Right handed': u'right',
    'Left handed': u'left',
    'Ambidextrous': u'ambidextrous',
    'Unknown': u'unknown',
}

SCORE_TYPES = (
    u'language',
    u'family',
    u'schizophrenic',
    u'dyslexic',
    u'dyscalculic',
    u'synaesthete',
)

SEQ_TYPES = (
    u'localizer_long_complex',
    u'localizer_long_easy',
    u'localizer_short_complex',
    u'localizer_short_easy',
)


###############################################################################
### MedicalExp entities #######################################################
###############################################################################
def import_subject(data_dir):
    """Import a subject from a data dir"""
    data, score_values = {}, []
    info = json.load(open('%s/subject.json' % data_dir))
    data['identifier'] = info['nip']
    # age varies with time: it should not be stored as an attribute of Subject
    # keep it for later use: store as an attribute of Assessment
    age_for_assessment = info['age']
    data['gender'] = GENDER_MAP.get(info['sex'], GENDER_MAP['3'])
    data['handedness'] = HANDEDNESS_MAP.get(info['laterality'],
                                            HANDEDNESS_MAP['Unknown'])
    for score in SCORE_TYPES:
        score_values.append({'name': score,
                             'value': info.get(score)})
    return data, age_for_assessment, score_values

def import_study(data_dir):
    """Import a study from a data dir"""
    data = {}
    data['data_filepath'] = osp.abspath(data_dir)
    data['name'] = u'localizer'
    data['description'] = u'localizer db'
    return data

def import_center(data_dir):
    """Import a center"""
    data = {}
    info = json.load(open('%s/subject.json' % data_dir))
    data['identifier'] = info['site']
    if info['site'] == u'SHFJ':
        data['name'] = u'SHFJ'
        data['department'] = u'Essonne'
        data['city'] = u'Orsay'
        data['country'] = u'France'
    elif info['site'] == u'Neurospin':
        data['name'] = u'Neurospin'
        data['department'] = u'Essonne'
        data['city'] = u'Gif-sur-Yvette'
        data['country'] = u'France'
    return data

def import_device(data_dir):
    """Import a device"""
    data = {}
    info = json.load(open('%s/subject.json' % data_dir))
    if info['site'] == u'Neurospin':
        data['name'] = '3T SIEMENS Trio'
        data['manufacturer'] = 'SIEMENS'
        data['model'] = 'Trio'
        data['hosted_by'] = 'Neurospin'
    if info['site'] == u'SHFJ':
        data['name'] = '3T Brucker'
        data['manufacturer'] = 'Brucker'
        data['model'] = '3T Brucker'
        data['hosted_by'] = 'SHFJ'
    return data

def import_assessment(data_dir, age_for_assessment, label, study_eid):
    """Import an assessment"""
    info = json.load(open('%s/subject.json' % data_dir))
    data = {}
    data['identifier'] = u'%s_%s' % (info['nip'], label)
    data['protocol'] = info['protocol']
    data['age_for_assessment'] = age_for_assessment
    data['timepoint'] = info['date']
    data['related_study'] = study_eid
    if info.get('date'):
        data['datetime'] = datetime.strptime(info['date'],
                                             '%Y-%m-%d %H:%M:%S')
    else:
        data['datetime'] = None
    return data


###############################################################################
### Neuroimaging entities #####################################################
###############################################################################
def import_neuroimaging(data_dir, dtype='anat', norm_prep=False):
    """Import a neuorimaging scan"""
    scan_data, mri_data = {}, {}
    info = json.load(open('%s/subject.json' % data_dir))
    # Label and id
    if dtype == 'anat':
        mri_data['sequence'] = u'T1'
        scan_data['identifier'] = u'%s_anat' % info['exam']
        scan_data['label'] = u'anatomy' if normalized else u'raw anatomy'
        scan_data['type'] = u'normalized T1' if norm_prep else u'raw T1'
        if norm_prep:
            scan_data['filepath'] = os.path.join(data_dir, 'anat', 'anat_defaced.nii.gz')
        else:
            scan_data['filepath'] = os.path.join(data_dir, 'anat', 'raw_anat_defaced.nii.gz')
    else:
        mri_data['sequence'] = u'EPI'
        scan_data['identifier'] = u'%s_fmri' % info['exam'] if norm_prep  \
                                  else u'%s_raw_fmri' % info['exam']
        scan_data['label'] = u'bold' if norm_prep else u'raw bold'
        scan_data['type'] = u'preprocessed fMRI' if norm_prep else u'raw fMRI'
        if norm_prep:
            scan_data['filepath'] = os.path.join(data_dir, 'fmri', 'bold.nii.gz')
        else:
            scan_data['filepath'] = os.path.join(data_dir, 'fmri', 'raw_bold.nii.gz')
    # Data properties
    scan_data['format'] = u'nii.gz'
    scan_data['timepoint'] = info['date']
    scan_data['completed'] = True
    scan_data['valid'] = True
    # Description
    if dtype == 'anat':
        scan_data['description'] = info['anatomy']
    else:
        scan_data['description'] = (
            u'epi_problem=%(epi_problem)s '
            'sound_problem=%(sound_problem)s '
            'video_problem=%(video_problem)s '
            'motor_error=%(motor_error)s' % info)
        for seq_type in SEQ_TYPES:
            if info[seq_type]:
                scan_data['description'] += u' %s' % seq_type
    # Update mri data
    mri_data.update(get_image_info(scan_data['filepath']))
    return scan_data, mri_data

def import_maps(data_dir, dtype='c'):
    """Import c/t maps"""
    info = json.load(open('%s/subject.json' % data_dir))
    base_path = os.path.join(data_dir, '%s_maps' % dtype)
    for img_path in glob.iglob(os.path.join(base_path, '*.nii.gz')):
        scan_data, mri_data = {}, {}
        scan_data['identifier'] = u'%s_%s_map' % (info['exam'], dtype)
        scan_data['label'] = unicode(os.path.split(img_path)[1].split(
            '.nii.gz')[0].replace('_', ' '))
        scan_data['format'] = u'nii.gz'
        scan_data['type'] = u'%s map' % dtype
        scan_data['filepath'] = img_path
        scan_data['timepoint'] = info['date']
        scan_data['completed'] = True
        scan_data['valid'] = True
        # Mri data
        mri_data['sequence'] = None
        mri_data.update(get_image_info(scan_data['filepath'], get_tr=False))
        name = os.path.split(img_path)[1].split('.nii.gz')[0]
        ext_resource = {}
        ext_resource['name'] = u'contrast definition'
        ext_resource['filepath'] = unicode(os.path.join(data_dir,
                                                        'contrasts',
                                                        '%s.json' % name))
        yield scan_data, mri_data, ext_resource

def import_mask(data_dir):
    """Import a mask"""
    scan_data, mri_data = {}, {}
    info = json.load(open('%s/subject.json' % data_dir))
    scan_data['identifier'] = u'%s_mask' % info['exam']
    scan_data['label'] = u'mask'
    scan_data['format'] = u'nii.gz'
    scan_data['type'] = u'boolean mask'
    scan_data['filepath'] = unicode(os.path.join(data_dir, 'mask.nii.gz'))
    scan_data['timepoint'] = info['date']
    scan_data['completed'] = True
    scan_data['valid'] = True
    mri_data['sequence'] = None
    mri_data.update(get_image_info(scan_data['filepath']))
    return scan_data, mri_data


###############################################################################
### Questionnaire entities ####################################################
###############################################################################
def import_questionnaire(data_dir):
    """Import a questionnaire and its questions"""
    questionnaire = {}
    questionnaire['name'] = u'localizer questionnaire'
    questionnaire['identifier'] = u'localizer_questionnaire'
    questionnaire['type'] = u'behavioural'
    questionnaire['version'] = u'1.0'
    questionnaire['language'] = u'French'
    behave = json.load(open('%s/behavioural.json' % data_dir))
    del behave['date']
    # Questions
    questions = []
    values = [behave[k] for k in sorted(behave.keys())]
    for i, (val, item) in enumerate(zip(values, sorted(behave.keys()))):
        question = {}
        question['identifier'] = u'localizer_%s' % i
        question['position'] = i
        question['text'] = unicode(item)
        question['type'] = u'boolean' if isinstance(val, bool) else u'float'
        question['possible_answers'] = None
        questions.append(question)
    return questionnaire, questions

def import_questionnaire_run(data_dir, questionnaire_id, questions_id):
    """Import a questionnaire run"""
    run = {}
    behave = json.load(open('%s/behavioural.json' % data_dir))
    sid = os.path.split(data_dir)[1]
    run['identifier'] = u'localizer_questionnaire_%s' % (sid)
    run['user_ident'] = u'subject'
    if behave.get('date'):
        run['datetime'] = datetime.strptime(
            behave['date'],
            '%Y-%m-%d %H:%M:%S')
    else:
        run['datetime'] = None
    del behave['date']
    del behave['nip']
    run['iteration'] = 1
    run['completed'] = True
    run['valid'] = True
    run['instance_of'] = questionnaire_id
    # Answers
    answers = []
    values = [behave[k] for k in sorted(behave.keys())]
    for i, (val, item) in enumerate(zip(values, sorted(behave.keys()))):
        answer = {}
        # XXX: handle str answers
        if not isinstance(val, (str, unicode)):
            answer['value'] = float(val) if val else None
            answer['datetime'] = run['datetime']
            answer['question'] = questions_id[item]
            answers.append(answer)
    return run, answers


###############################################################################
### Genomics entities #########################################################
###############################################################################
# XXX These functions may be pushed in helpers, as they may be more general
def import_genomic_measures(measure_path, genetics_basename):
    """Import a genomic measures"""
    g_measures = {}
    # path to BED / BIM / FAM files
    bed_path = os.path.join(measure_path, genetics_basename + '.bed')
    bim_path = os.path.join(measure_path, genetics_basename + '.bim')
    fam_path = os.path.join(measure_path, genetics_basename + '.fam')
    # read FAM file as CSV file
    fam_file = open(fam_path, 'rU')
    fam_reader = csv.reader(fam_file, delimiter=' ')
    # one subject per line
    for row in fam_reader:
        subject_id = row[1]
        genomic_measure = {}
        genomic_measure['identifier'] = u'genomic_measure_%s' % subject_id
        genomic_measure['type'] = u'SNP'
        genomic_measure['format'] = u'plink'
        genomic_measure['filepath'] = unicode(bed_path)
        genomic_measure['chip_serialnum'] = None
        genomic_measure['completed'] = True
        genomic_measure['valid'] = True
        genomic_measure['platform'] = None
        g_measures[subject_id] = genomic_measure
    return g_measures


###############################################################################
### MAIN ######################################################################
###############################################################################
if __name__ == '__main__':
    # Create store
    from cubicweb.dataimport import SQLGenObjectStore
    store = SQLGenObjectStore(session)
    sqlgen_store = True

    root_dir = osp.abspath(sys.argv[4])
    subjects_dir = osp.join(root_dir, 'subjects')
    genetics_dir = osp.join(root_dir, 'genetics')

    ### Study #################################################################
    study = import_study(data_dir=root_dir)
    study = store.create_entity('Study', **study)

    ### Initialize questionnaire ##############################################
    one_subject = glob.glob('%s/*' % subjects_dir)[0]
    questionnaire, questions = import_questionnaire(one_subject)
    questionnaire = store.create_entity('Questionnaire', **questionnaire)
    questions_id = {}
    for question in questions:
        question['questionnaire'] = questionnaire.eid
        question = store.create_entity('Question', **question)
        questions_id[question.text] = question.eid

    ### Initialize genetics ####################################################
    # Chromosomes
    chrs = import_chromosomes(os.path.join(genetics_dir, 'chromosomes.json'))
    chr_map = {}
    for _chr in chrs:
        print 'chr', _chr['name']
        _chr = store.create_entity('Chromosome', **_chr)
        chr_map.setdefault(_chr['name'], _chr.eid)
    # Genes
    genes = import_genes(os.path.join(genetics_dir, 'chromosomes.json'),
                         os.path.join(genetics_dir, 'hg18.refGene.meta'))
    for gene in genes:
        print 'gene', gene['name'], gene['chromosome']
        gene['chromosome'] = chr_map[gene['chromosome']]
        gene = store.create_entity('Gene', **gene)
    # Flush/Commit
    if sqlgen_store:
        store.flush()
    # Snps
    snps = import_snps(os.path.join(genetics_dir, 'chromosomes.json'),
                       os.path.join(genetics_dir, 'Localizer94.bim'))
    snp_eids = []
    for ind, snp in enumerate(snps):
        print 'snp', snp['rs_id']
        snp['chromosome'] = chr_map[snp['chromosome']]
        snp = store.create_entity('Snp', **snp)
        snp_eids.append(snp.eid)
        if sqlgen_store and ind and ind % 100000 == 0:
            store.flush()
    # Flush/Commit
    if sqlgen_store:
        store.flush()
    # Platform
    platform = {'identifier': 'Affymetrix_6.0'}
    platform = store.create_entity('GenomicPlatform', **platform)
    for snp_eid in snp_eids:
        store.relate(platform.eid, 'related_snps', snp_eid)

    ### Genetics measures #####################################################
    gen_measures = import_genomic_measures(genetics_dir, 'Localizer94')

    # Flush/Commit
    if sqlgen_store:
        store.flush()

    ###########################################################################
    ### Subjects ##############################################################
    ###########################################################################
    centers, devices, score_defs = {}, {}, {}
    for sid in glob.glob('%s/*' % subjects_dir):

        print '-------->', sid, os.path.split(sid)[1]

        # Centers #############################################################
        center = import_center(sid)
        if center['name'] not in centers:
            center = store.create_entity('Center', **center)
            centers.setdefault(center.name, center.eid)
            center_eid = center.eid
        else:
            center_eid = centers[center['name']]

        # Devices #############################################################
        device = import_device(sid)
        device['hosted_by'] = centers[device['hosted_by']]
        if device['name'] not in devices:
            device = store.create_entity('Device', **device)
            devices.setdefault(device.name, device.eid)
            device_id = device.eid
        else:
            device_id = devices[device['name']]

        # Subject #############################################################
        subject, age_for_assessment, score_values = import_subject(sid)
        subject = store.create_entity('Subject', **subject)
        store.relate(subject.eid, 'related_studies', study.eid)
        for score_val in score_values:
            value = score_val['value']
            if not value:
                continue
            if score_val['name'] in score_defs:
                def_eid = score_defs.get(score_val['name'])
            else:
                score_def = {}
                score_def['name'] = score_val['name']
                score_def['category'] = u'demographics'
                score_def['type'] = u'string'
                score_def = store.create_entity('ScoreDefinition', **score_def)
                score_defs[score_val['name']] = score_def.eid
                def_eid = score_def.eid
            score_val = store.create_entity('ScoreValue', definition=def_eid,
                                            text=value)
            store.relate(subject.eid, 'related_infos', score_val.eid)

        # Design matrix #######################################################
        dm_res = store.create_entity('ExternalResource',
                                     name=u'design_matrix',
                                     related_study=study.eid,
                                     filepath=unicode(osp.relpath(
                                         os.path.join(sid, 'design_matrix.json'),
                                         start=root_dir)))

        # Genetics ############################################################
        gen_assessment = import_assessment(sid, age_for_assessment, 'genetics', study.eid)
        gen_assessment = store.create_entity('Assessment', **gen_assessment)
        store.relate(center_eid, 'holds', gen_assessment.eid)
        store.relate(subject.eid, 'concerned_by', gen_assessment.eid)
        measure = gen_measures[subject.identifier]
        measure['platform'] = platform.eid
        measure['related_study'] = study.eid
        measure['filepath'] = osp.relpath(measure['filepath'], start=root_dir)
        measure = store.create_entity('GenomicMeasure', **measure)
        store.relate(measure.eid, 'concerns', subject.eid, subjtype='GenomicMeasure')
        store.relate(gen_assessment.eid, 'generates', measure.eid, subjtype='Assessment')

        # Anat & fMRI ############################################################
        # anat assessment
        anat_assessment = import_assessment(sid, age_for_assessment, 'anat', study.eid)
        anat_assessment = store.create_entity('Assessment', **anat_assessment)
        store.relate(center_eid, 'holds', anat_assessment.eid)
        store.relate(subject.eid, 'concerned_by', anat_assessment.eid)
        for normalized in (False, True):
            scan_anat, mri_anat = import_neuroimaging(sid, 'anat', normalized)
            mri_anat = store.create_entity('MRIData', **mri_anat)
            scan_anat['has_data'] = mri_anat.eid
            scan_anat['related_study'] = study.eid
            # Get the relative filepath
            scan_anat['filepath'] = osp.relpath(scan_anat['filepath'], start=root_dir)
            scan_anat = store.create_entity('Scan', **scan_anat)
            store.relate(scan_anat.eid, 'concerns', subject.eid, subjtype='Scan')
            store.relate(scan_anat.eid, 'uses_device', device_id)
            store.relate(anat_assessment.eid, 'generates', scan_anat.eid, subjtype='Assessment')
        # fmri assessment
        fmri_assessment = import_assessment(sid, age_for_assessment, 'fmri', study.eid)
        fmri_assessment = store.create_entity('Assessment', **fmri_assessment)
        store.relate(center_eid, 'holds', fmri_assessment.eid)
        store.relate(subject.eid, 'concerned_by', fmri_assessment.eid)
        for preprocessed in (False, True):
            scan_fmri, mri_fmri = import_neuroimaging(sid, 'fmri', preprocessed)
            mri_fmri = store.create_entity('MRIData', **mri_fmri)
            scan_fmri['has_data'] = mri_fmri.eid
            scan_fmri['related_study'] = study.eid
            # Get the relative filepath
            scan_fmri['filepath'] = osp.relpath(scan_fmri['filepath'], start=root_dir)
            scan_fmri = store.create_entity('Scan', **scan_fmri)
            store.relate(scan_fmri.eid, 'concerns', subject.eid, subjtype='Scan')
            store.relate(scan_fmri.eid, 'uses_device', device_id)
            store.relate(fmri_assessment.eid, 'generates', scan_fmri.eid, subjtype='Assessment')

        # c-maps & t-maps #####################################################
        for dtype, label in (('c', 'c_maps'), ('t', 't_maps')):
            assessment = import_assessment(sid, age_for_assessment, label, study.eid)
            assessment = store.create_entity('Assessment', **assessment)
            store.relate(center_eid, 'holds', assessment.eid)
            store.relate(subject.eid, 'concerned_by', assessment.eid)
            for scan, mri, con_res in import_maps(sid, dtype):
                mri = store.create_entity('MRIData', **mri)
                if con_res:
                    con_res['related_study'] = study.eid
                    con_res['filepath'] = osp.relpath(con_res['filepath'], start=root_dir)
                    con_res = store.create_entity('ExternalResource', **con_res)
                scan['has_data'] = mri.eid
                scan['related_study'] = study.eid
                # Get the relative filepath
                scan['filepath'] = osp.relpath(scan['filepath'], start=root_dir)
                scan = store.create_entity('Scan', **scan)
                store.relate(scan.eid, 'concerns', subject.eid, subjtype='Scan')
                store.relate(scan.eid, 'uses_device', device_id)
                store.relate(assessment.eid, 'generates', scan.eid, subjtype='Assessment')
                store.relate(scan.eid, 'external_resources', con_res.eid)
                store.relate(scan.eid, 'external_resources', dm_res.eid)

        # mask ################################################################
        assessment = import_assessment(sid, age_for_assessment, 'mask', study.eid)
        assessment = store.create_entity('Assessment', **assessment)
        store.relate(center_eid, 'holds', assessment.eid)
        store.relate(subject.eid, 'concerned_by', assessment.eid)
        scan, mri = import_mask(sid)
        mri = store.create_entity('MRIData', **mri)
        scan['has_data'] = mri.eid
        scan['related_study'] = study.eid
        # Get the relative filepath
        scan['filepath'] = osp.relpath(scan['filepath'], start=root_dir)
        scan = store.create_entity('Scan', **scan)
        store.relate(scan.eid, 'concerns', subject.eid, subjtype='Scan')
        store.relate(scan.eid, 'uses_device', device_id)
        store.relate(assessment.eid, 'generates', scan.eid, subjtype='Assessment')

        # Questionnaire run ###################################################
        assessment = import_assessment(sid, age_for_assessment, 'questionnaire', study.eid)
        assessment = store.create_entity('Assessment', **assessment)
        store.relate(center_eid, 'holds', assessment.eid)
        store.relate(subject.eid, 'concerned_by', assessment.eid)
        run, answers = import_questionnaire_run(sid, questionnaire.eid, questions_id)
        run['related_study'] = study.eid
        run = store.create_entity('QuestionnaireRun', **run)
        # Answers
        for answer in answers:
            answer['questionnaire_run'] = run.eid
            answer = store.create_entity('Answer', **answer)
        store.relate(run.eid, 'concerns', subject.eid, subjtype='QuestionnaireRun')
        store.relate(assessment.eid, 'generates', run.eid, subjtype='Assessment')

    # Flush/Commit
    if sqlgen_store:
        store.flush()
    store.commit()



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-brainomics schema"""
from cubicweb.schemas.base import CWUser
from yams.buildobjs import (EntityType, RelationDefinition, SubjectRelation, String, RichString,
                            Int, Float, Boolean)

from cubes.questionnaire.schema import QuestionnaireRun, Questionnaire, Question
from cubes.neuroimaging.schema import Scan, AnatomicalRegion
from cubes.genomics.schema import GenomicMeasure, ColumnRef
from cubes.medicalexp.schema  import Assessment, ProcessingRun, ScoreDefinition, ScoreValue

from cubes.brainomics import COMMENTED_ENTITIES, WIKI_ENTITIES


# ProcessingRun -> Measure
ProcessingRun.add_relation(SubjectRelation('QuestionnaireRun', cardinality='**'), name='inputs')
ProcessingRun.add_relation(SubjectRelation('Scan', cardinality='**'), name='inputs')
ProcessingRun.add_relation(SubjectRelation('GenomicMeasure', cardinality='**'), name='inputs')

ProcessingRun.add_relation(SubjectRelation('QuestionnaireRun', cardinality='**'), name='outputs')
ProcessingRun.add_relation(SubjectRelation('Scan', cardinality='**'), name='outputs')
ProcessingRun.add_relation(SubjectRelation('GenomicMeasure', cardinality='**'), name='outputs')

# Maesure -> Subject
QuestionnaireRun.add_relation(SubjectRelation(('Subject', 'Group'), cardinality='1*', inlined=True), name='concerns')
Scan.add_relation(SubjectRelation(('Subject', 'Group'), cardinality='1*', inlined=True), name='concerns')
GenomicMeasure.add_relation(SubjectRelation(('Subject', 'Group'), cardinality='1*', inlined=True), name='concerns')

# Assessment -> Measure
Assessment.add_relation(SubjectRelation('QuestionnaireRun', cardinality='**'), name='uses')
Assessment.add_relation(SubjectRelation('Scan', cardinality='**'), name='uses')
Assessment.add_relation(SubjectRelation('GenomicMeasure', cardinality='**'), name='uses')

Assessment.add_relation(SubjectRelation('QuestionnaireRun', cardinality='?*'), name='generates')
Assessment.add_relation(SubjectRelation('Scan', cardinality='?*'), name='generates')
Assessment.add_relation(SubjectRelation('GenomicMeasure', cardinality='?*'), name='generates')

# Score -> Measure
ScoreValue.add_relation(SubjectRelation('QuestionnaireRun', cardinality='?*'), name='measure')
ScoreValue.add_relation(SubjectRelation('Scan', cardinality='?*'), name='measure')
ScoreValue.add_relation(SubjectRelation('GenomicMeasure', cardinality='?*'), name='measure')

# Use for filepath computation
QuestionnaireRun.add_relation(SubjectRelation('Study', cardinality='1*', inlined=True), name='related_study')
Scan.add_relation(SubjectRelation('Study', cardinality='1*', inlined=True), name='related_study')
GenomicMeasure.add_relation(SubjectRelation('Study', cardinality='1*', inlined=True), name='related_study')
QuestionnaireRun.add_relation(SubjectRelation('Study', cardinality='**'), name='other_studies')
Scan.add_relation(SubjectRelation('Study', cardinality='**'), name='other_studies')
GenomicMeasure.add_relation(SubjectRelation('Study', cardinality='**'), name='other_studies')

# Device -> Measure
QuestionnaireRun.add_relation(SubjectRelation('Device', cardinality='?*', inlined=True), name='uses_device')
Scan.add_relation(SubjectRelation('Device', cardinality='?*', inlined=True), name='uses_device')
GenomicMeasure.add_relation(SubjectRelation('Device', cardinality='?*', inlined=True), name='uses_device')

# Measure -> ExternalResources
# Questionnaire may have some specifications files
Questionnaire.add_relation(SubjectRelation('ExternalResource', cardinality='*1'), name='external_resources')
# Some questionnaire run could have specific results files (subject dependant)
QuestionnaireRun.add_relation(SubjectRelation('ExternalResource', cardinality='*1'), name='external_resources')
Scan.add_relation(SubjectRelation('ExternalResource', cardinality='*1'), name='external_resources')
GenomicMeasure.add_relation(SubjectRelation('ExternalResource', cardinality='*1'), name='external_resources')

# Various relations
Questionnaire.add_relation(SubjectRelation('ScoreDefinition', cardinality='*?'), name='definitions')
ScoreDefinition.add_relation(SubjectRelation('Question', cardinality='**'), name='used_by')
AnatomicalRegion.add_relation(SubjectRelation('ScoreValue', cardinality='**'), name='concerned_by')
ColumnRef.add_relation(SubjectRelation('Assessment', cardinality='1*', inlined=True), name='assessment')

# Extra fields for CWUser

CWUser.add_relation(String(maxsize=512, fulltextindexed=True), name='affiliation')
CWUser.add_relation(String(maxsize=512, fulltextindexed=True), name='extra_infos')


# Comments for entities
class comments(RelationDefinition):
    subject = 'Comment'
    object = COMMENTED_ENTITIES

# Links to "Wikis" / Cards
class wiki(RelationDefinition):
    subject = WIKI_ENTITIES
    object = 'Card'
    cardinality = '**'



# -*- coding: utf-8 -*-
# copyright 2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
from random import randrange
from optparse import OptionParser
from datetime import timedelta, datetime

import numpy.random as nr

from cubicweb.dataimport import SQLGenObjectStore

from cubes.brainomics.importers.helpers import (get_image_info, import_genes,
                                                import_chromosomes, import_snps)


###############################################################################
### DATA GENERATION CONFIGURATION #############################################
###############################################################################
NB_SUBJECTS = 10000
NB_SUBJECTS_GROUPS = 5
AGE_DATETIMES = (datetime.strptime('1/1/1930 1:30 PM', '%m/%d/%Y %I:%M %p'),
                 datetime.strptime('1/1/1990 4:50 AM', '%m/%d/%Y %I:%M %p'))
PROJECT_DATETIMES = (datetime.strptime('1/1/2010 1:30 PM', '%m/%d/%Y %I:%M %p'),
                     datetime.strptime('1/1/2013 4:50 AM', '%m/%d/%Y %I:%M %p'))
SEX = {0: u'male', 1: u'female'}
HANDEDNESS = {0: u'right', 1: u'left', 2: u'ambidextrous', 3: u'mixed'}


###############################################################################
### UTILITY FUNCTIONS #########################################################
###############################################################################
def random_date(_type=None):
    """
    This function will return a random datetime between two datetime
    objects.
    From http://stackoverflow.com/questions/553303/generate-a-random-date-between-two-other-dates
    """
    if _type=='age':
        start, end = AGE_DATETIMES[0], AGE_DATETIMES[1]
    else:
        start, end = PROJECT_DATETIMES[0], PROJECT_DATETIMES[1]
    delta = end - start
    int_delta = (delta.days * 24 * 60 * 60) + delta.seconds
    random_second = randrange(int_delta)
    return (start + timedelta(seconds=random_second))

def import_questionnaire(store, subject_eid, center_eid, study_eid,
                         questionnaire_eid, questions, label):
    date = random_date()
    assessment = store.create_entity('Assessment', identifier=u'%s_%s' % (label, subject_eid),
                                     datetime=date, protocol=u'Demo questionnaire')
    store.relate(assessment.eid, 'related_study', study_eid, subtype='Assessment')
    store.relate(center_eid, 'holds', assessment.eid)
    store.relate(subject_eid, 'concerned_by', assessment.eid)
    user_ident = nr.randint(10)
    user_ident = u'parent' if user_ident>8 else u'subject'
    q_qrun = store.create_entity('QuestionnaireRun', identifier=u'%s_%s' % (label, subject_eid),
                                   user_ident=user_ident, datetime=date,
                                   iteration=1, completed=True, valid=True,
                                   instance_of=questionnaire_eid)
    store.relate(q_qrun.eid, 'related_study', study_eid)
    store.relate(q_qrun.eid, 'concerns', subject_eid, subjtype='QuestionnaireRun')
    store.relate(assessment.eid, 'generates', q_qrun.eid, subjtype='Assessment')
    for name, (question, _min, _max) in questions.iteritems():
        score_value = store.create_entity('Answer', value=nr.randint(_min, _max),
                                          question=question, datetime=date,
                                          questionnaire_run=q_qrun.eid)

def import_genomic(store, subject_eid, center_eid, study_eid, platform=None):
    date = random_date()
    assessment = store.create_entity('Assessment', identifier=u'genomics_%s' % subject_eid,
                                     datetime=date, protocol=u'Demo genomics')
    store.relate(assessment.eid, 'related_study', study_eid, subjtype='Assessment')
    store.relate(center_eid, 'holds', assessment.eid)
    store.relate(subject_eid, 'concerned_by', assessment.eid)
    measure = store.create_entity('GenomicMeasure', identifier=u'genomic_measure_%s' % subject_eid,
                                  type=u'SNP', format=u'plink',
                                  platform=platform if platform else None,
                                  filepath=u'%s/genomics.tar.gz' % subject_eid,
                                  completed=True, valid=True, related_study=study_eid)
    store.relate(measure.eid, 'concerns', subject_eid, subjtype='GenomicMeasure')
    store.relate(assessment.eid, 'generates', measure.eid, subjtype='Assessment')

def import_anat_images(store, subject_eid, center_eid, study_eid, mri_1, mri_2):
    date = random_date()
    assessment = store.create_entity('Assessment', identifier=u'anat_%s' % subject_eid,
                                     datetime=date, protocol=u'Demo anat')
    store.relate(assessment.eid, 'related_study', study_eid, subjtype='Assessment')
    store.relate(center_eid, 'holds', assessment.eid)
    store.relate(subject_eid, 'concerned_by', assessment.eid)
    mri_data = store.create_entity('MRIData',
                                   shape_x=128, shape_y=128, shape_z=64,
                                   sequence=u'T1',
                                   voxel_res_x=1.5, voxel_res_y=1.5, voxel_res_z=1.5).eid
    scan_data = store.create_entity('Scan', has_data=mri_data,
                                    related_study=study_eid,identifier=u'anat_%s' % subject_eid,
                                    label=u'anatomy', type=u'normalized T1',
                                    format=u'nii.gz',
                                    completed=True, valid=True,
                                    filepath=u'%s/anat.nii.gz' % subject_eid).eid
    store.relate(scan_data, 'concerns', subject_eid, subjtype='Scan')
    device = nr.randint(1)
    store.relate(scan_data, 'uses_device', mri_1 if device else mri_2)
    store.relate(assessment.eid, 'generates', scan_data, subjtype='Assessment')

def import_fmri_images(store, subject_eid, center_eid, study_eid, mri_1, mri_2):
    date = random_date()
    assessment = store.create_entity('Assessment', identifier=u'fmri_%s' % subject_eid,
                                     datetime=date, protocol=u'Demo fmri')
    store.relate(assessment.eid, 'related_study', study_eid, subjtype='Assessment')
    store.relate(center_eid, 'holds', assessment.eid)
    store.relate(subject_eid, 'concerned_by', assessment.eid)
    mri_data = store.create_entity('MRIData',
                                   sequence=u'EPI',
                                   shape_x=128, shape_y=128, shape_z=64, shape_t=256,
                                   voxel_res_x=1.5, voxel_res_y=1.5, voxel_res_z=1.5).eid
    scan_data = store.create_entity('Scan', has_data=mri_data,
                                    related_study=study_eid,identifier=u'fmri_%s' % subject_eid,
                                    label=u'bold', type=u'preprocessed fMRI',
                                    format=u'nii.gz',
                                    completed=True, valid=True,
                                    filepath=u'%s/bold.nii.gz' % subject_eid).eid
    store.relate(scan_data, 'concerns', subject_eid, subjtype='Scan')
    device = nr.randint(1)
    store.relate(scan_data, 'uses_device', mri_1 if device else mri_2)
    store.relate(assessment.eid, 'generates', scan_data, subjtype='Assessment')

def import_constrat_images(store, subject_eid, center_eid, study_eid, mri_1, mri_2):
    date = random_date()
    assessment = store.create_entity('Assessment', identifier=u'cmap_%s' % subject_eid,
                                     datetime=date, protocol=u'Demo c map')
    store.relate(assessment.eid, 'related_study', study_eid, subjtype='Assessment')
    store.relate(center_eid, 'holds', assessment.eid)
    store.relate(subject_eid, 'concerned_by', assessment.eid)
    for ind, label in enumerate((u'Audio-Video', u'Text reading', u'Video reading',
                                 u'Social behavior', u'Audio interaction')):
        for smap in (u'c map', u't map'):
            mri_data = store.create_entity('MRIData',
                                           sequence=u'EPI',
                                           shape_x=128, shape_y=128, shape_z=64,
                                           voxel_res_x=1.5, voxel_res_y=1.5, voxel_res_z=1.5).eid
            scan_data = store.create_entity('Scan', has_data=mri_data,
                                            related_study=study_eid,
                                            identifier=u'%s_%s_%s' % (smap.replace(' ', ''),
                                                                      ind, subject_eid),
                                            label=label, type=smap,
                                            format=u'nii.gz',
                                            completed=True, valid=True,
                                            filepath=u'%s/%s_%s.nii.gz' % (smap.replace(' ', ''),
                                                                           subject_eid, ind)).eid
            store.relate(scan_data, 'concerns', subject_eid, subjtype='Scan')
            store.relate(assessment.eid, 'generates', scan_data, subjtype='Assessment')


###############################################################################
### IMPORT ####################################################################
###############################################################################
if __name__ == '__main__':
    parser = OptionParser()
    parser.add_option("-c", "--chromosomes", dest="chromosomes",
                      help="Json file of chromosomes")
    parser.add_option("-g", "--genes", dest="genes",
                      help="Json file of genes")
    parser.add_option("-s", "--snps", dest="snps",
                      help="Json file of snps")
    (options, args) = parser.parse_args(__args__)

    store = SQLGenObjectStore(session)


    ###############################################################################
    ### GENETICS DATA #############################################################
    ###############################################################################
    platform = None
    if options.chromosomes:
        chrs = import_chromosomes(options.chromosomes)
        chr_map = {}
        for _chr in chrs:
            print 'chr', _chr['name']
            _chr = store.create_entity('Chromosome', **_chr)
            chr_map.setdefault(_chr['name'], _chr.eid)
        store.flush()
        if options.genes:
            genes = import_genes(options.chromosomes, options.genes)
            for gene in genes:
                print 'gene', gene['name'], gene['chromosome']
                gene['chromosome'] = chr_map[gene['chromosome']]
                gene = store.create_entity('Gene', **gene)
            store.flush()
        if options.snps:
            snps = import_snps(options.chromosomes, options.snps)
            snp_eids = []
            for ind, snp in enumerate(snps):
                print 'snp', snp['rs_id']
                snp['chromosome'] = chr_map[snp['chromosome']]
                snp = store.create_entity('Snp', **snp)
                snp_eids.append(snp.eid)
                if ind and ind % 100000 == 0:
                    store.flush()
            store.flush()
        # Platform
        platform = {'identifier': 'Affymetrix_6.0'}
        platform = store.create_entity('GenomicPlatform', **platform).eid
        for snp_eid in snp_eids:
            store.relate(platform, 'related_snps', snp_eid)
        store.flush()

    ###############################################################################
    ### STUDY/PI/CENTERS ##########################################################
    ###############################################################################
    study = store.create_entity('Study', name=u'Demo study',
                                data_filepath=u'/tmp/demo_data',
                                description=u'Demo study with random data',
                                keywords=u'demo;random;semantic').eid
    pi_1 = store.create_entity('Investigator', identifier=u'pi_1',
                               firstname=u'Angus', lastname=u'Young', title=u'PhD',
                               institution=u'Research Center', department=u'Cognitive Sciences').eid
    pi_2 = store.create_entity('Investigator', identifier=u'pi_2',
                               firstname=u'Malcom', lastname=u'Young', title=u'MD',
                               institution=u'Research Center', department=u'Cognitive Sciences').eid
    pi_3 = store.create_entity('Investigator', identifier=u'pi_3',
                               firstname=u'Brian', lastname=u'Jonhson', title=u'PhD',
                               institution=u'Research Center', department=u'IT').eid
    pi_4 = store.create_entity('Investigator', identifier=u'pi_4',
                               firstname=u'Phil', lastname=u'Rudd', title=u'PhD',
                               institution=u'Research Center', department=u'IT').eid
    center = store.create_entity('Center', identifier=u'center', name=u'Demo center',
                                 department=u'Medical research', city=u'Paris',
                                 country=u'France').eid
    mri_1 = store.create_entity('Device', name=u'Magnetom', model=u'Trio',
                                manufacturer=u'Siemens',
                                serialnum=1234, hosted_by=center).eid
    mri_2 = store.create_entity('Device', name=u'Ingenia', model=u'3.0 T',
                                manufacturer=u'Philips',
                                serialnum=4321, hosted_by=center).eid


    ###############################################################################
    ### SCOREDEFS #################################################################
    ###############################################################################
    score_defs = {}
    possible_values = u'0 = Control; 1 = Autism; 2 = Aspergers; 3 = PDD-NOS; 4 = Aspergers or PDD-NOS'
    score_defs['dsm_iv_tr'] = (store.create_entity('ScoreDefinition', name=u'DSM-IV-TR Diagnostic Category', category=u'biological', type=u'numerical', possible_values=possible_values).eid, 0, 4)
    score_defs['scq_total'] = (store.create_entity('ScoreDefinition', name=u'Social Communication Questionnaire Total', category=u'behavioral', type=u'numerical', possible_values=u'0-39').eid, 0, 39)
    score_defs['aq_total'] = (store.create_entity('ScoreDefinition', name=u'Total Raw Score of the Autism Quotient', category=u'behavioral', type=u'numerical', possible_values=u'0-50').eid, 0, 50)
    score_defs['fiq_WASI'] = (store.create_entity('ScoreDefinition', name=u'FIQ Standard Score - WASI', category=u'FIQ', type=u'numerical').eid, 50, 160)
    score_defs['viq_WASI'] = (store.create_entity('ScoreDefinition', name=u'VIQ Standard Score - WASI', category=u'VIQ', type=u'numerical').eid, 55, 160)
    score_defs['piq_WASI'] = (store.create_entity('ScoreDefinition', name=u'PIQ Standard Score - WASI', category=u'PIQ', type=u'numerical').eid, 53, 160)


    store.flush()


    ###############################################################################
    ### QUESTIONNAIRES ############################################################
    ###############################################################################
    # ADOS
    name = u'Autism Diagnostic Observation Schedule Module (ADOS)'
    ados_questionnaire_eid = store.create_entity('Questionnaire', name=name, language=u'en', version=u'v 0.2.1', identifier=u'ados', type=u'behavioral').eid
    ados_questions = {}
    text = u'Autism Diagnostic Observation Schedule Module'
    ados_questions['ados_module'] = (store.create_entity('Question', identifier=u'ados_module', position=0, text=text, type=u'numerical', questionnaire=ados_questionnaire_eid, possible_answers=u'1-4').eid, 1, 4)
    text = u'Classical Total ADOS Score (Communication subscore + Social Interaction subscore)'
    ados_questions['ados_total'] = (store.create_entity('Question', identifier=u'ados_total', position=1, text=text, type=u'numerical', questionnaire=ados_questionnaire_eid, possible_answers=u'0-22').eid, 0, 22)
    text = u'Communication Total Subscore of the Classic ADOS'
    ados_questions['ados_comm'] = (store.create_entity('Question', identifier=u'ados_comm', position=2, text=text, type=u'numerical', questionnaire=ados_questionnaire_eid, possible_answers=u'0-8').eid, 0, 8)
    text = u'Social Total Subscore of the Classic ADOS'
    ados_questions['ados_social'] = (store.create_entity('Question', identifier=u'ados_social', position=3, text=text, type=u'numerical', questionnaire=ados_questionnaire_eid, possible_answers=u'0-14').eid, 0, 14)
    text = u'Stereotyped Behaviors and Restricted Interests Total Subscore of the Classic ADOS'
    ados_questions['ados_stereo_behav'] = (store.create_entity('Question', identifier=u'ados_stereo_behav', position=4, text=text, type=u'numerical', questionnaire=ados_questionnaire_eid, possible_answers=u'0-8').eid, 0, 8)
    text = u'Was ADOS scored and administered by research reliable personnel?'
    possible_answers = u'0 = not research reliable; 1 = research reliable'
    ados_questions['ados_rsrch_reliable'] = (store.create_entity('Question', identifier=u'ados_rsrch_reliable', position=5, text=text, type=u'boolean', questionnaire=ados_questionnaire_eid, possible_answers=possible_answers).eid, 0, 1)
    text = u'Social Affect Total Subscore for Gotham Algorithm of the ADOS'
    ados_questions['ados_gotham_soc_affect'] = (store.create_entity('Question', identifier=u'ados_gotham_soc_affect', position=6, text=text, type=u'numerical', questionnaire=ados_questionnaire_eid, possible_answers=u'0-20').eid, 0, 20)
    text = u'Restrictive and Repetitive Behaviors Total Subscore for Gotham Algorithm of the ADOS'
    ados_questions['ados_gotham_rrb'] = (store.create_entity('Question', identifier=u'ados_gotham_rrb', position=7, text=text, type=u'numerical', questionnaire=ados_questionnaire_eid, possible_answers=u'0-8').eid, 0, 8)
    text = u'Social Affect Total + Restricted and Repetitive Behaviors Total'
    ados_questions['ados_gotham_total'] = (store.create_entity('Question', identifier=u'ados_gotham_total', position=8, text=text, type=u'numerical', questionnaire=ados_questionnaire_eid, possible_answers=u'0-28').eid, 0, 28)
    text = u'Individually Calibrated Severity Score for Gotham Algorithm of the ADOS'
    ados_questions['ados_gotham_severity'] = (store.create_entity('Question', identifier=u'ados_gotham_severity', position=9, text=text, type=u'numerical', questionnaire=ados_questionnaire_eid, possible_answers=u'1-10').eid, 1, 10)

    # VINELAND
    vineland_questionnaire_eid = store.create_entity('Questionnaire', name=u'Vineland Adaptive Behavior Scales', identifier=u'vineland', language=u'fr', version=u'v 0.1.0', type=u'behavioral').eid
    vineland_questions = {}
    text = u'Vineland Adaptive Behavior Scales Receptive Language V Scaled Score'
    vineland_questions['vineland_receptive_v_scaled'] = (store.create_entity('Question', identifier=u'vineland_receptive_v_scaled', position=0, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'1-24').eid, 1, 24)
    text = u'Vineland Adaptive Behavior Scales Expressive Language V Scaled Score'
    vineland_questions['vineland_expressive_v_scaled'] = (store.create_entity('Question', identifier=u'vineland_expressive_v_scaled', position=1, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'1-24').eid, 1, 24)
    text = u'Vineland Adaptive Behavior Scales Written Language V Scaled Score'
    vineland_questions['vineland_written_v_scaled'] = (store.create_entity('Question', identifier=u'vineland_written_v_scaled', position=2, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'1-24').eid, 1, 24)
    text = u'Vineland Adaptive Behavior Scales Communication Standard Score'
    vineland_questions['vineland_communication_standard'] = (store.create_entity('Question', identifier=u'vineland_communication_standard', position=3, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'20-160').eid, 20, 160)
    text = u'Vineland Adaptive Behavior Scales Personal Daily Living Skills V Scaled Score'
    vineland_questions['vineland_personal_v_scaled'] = (store.create_entity('Question', identifier=u'vineland_personal_v_scaled', position=4, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'1-24').eid, 1, 24)
    text = u'Vineland Adaptive Behavior Scales Domestic Daily Living Skills V Scaled Score'
    vineland_questions['vineland_domestic_v_scaled'] = (store.create_entity('Question', identifier=u'vineland_domestic_v_scaled', position=5, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'1-24').eid, 1, 24)
    text = u'Vineland Adaptive Behavior Scales Community Daily Living Skills V Scaled Score'
    vineland_questions['vineland_community_v_scaled'] = (store.create_entity('Question', identifier=u'vineland_community_v_scaled', position=6, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'1-24').eid, 1, 24)
    text = u'Vineland Adaptive Behavior Scales Daily Living Skills Standard Score'
    vineland_questions['vineland_dailylvng_standard'] = (store.create_entity('Question', identifier=u'vineland_dailylvng_standard', position=7, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'20-160').eid, 20, 160)
    text = u'Vineland Adaptive Behavior Scales Interpersonal Relationships V Scaled Score'
    vineland_questions['vineland_interpersonal_v_scaled'] = (store.create_entity('Question', identifier=u'vineland_interpersonal_v_scaled', position=8, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'1-24').eid, 1, 24)
    text = u'Vineland Adaptive Behavior Scales Play and Leisure Time V Scaled Score'
    vineland_questions['vineland_play_v_scaled'] = (store.create_entity('Question', identifier=u'vineland_play_v_scaled', position=9, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'1-24').eid, 1, 24)
    text = u'Vineland Adaptive Behavior ScalesCoping Skills V Scaled Score'
    vineland_questions['vineland_coping_v_scaled'] = (store.create_entity('Question', identifier=u'vineland_coping_v_scaled', position=10, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'1-24').eid, 1, 24)
    text = u'Vineland Adaptive Behavior Scales Socialization Standard Score'
    vineland_questions['vineland_social_standard'] = (store.create_entity('Question', identifier=u'vineland_social_standard', position=11, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'20-160').eid, 20, 160)
    text = u'Sum of Vineland Standard Scores (Communication + Daily Living Skills + Socialization)'
    vineland_questions['vineland_sum_scores'] = (store.create_entity('Question', identifier=u'vineland_sum_scores', position=12, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'76-480').eid, 76, 480)
    text = u'Vineland Adaptive Behavior Composite Standard score'
    vineland_questions['vineland_abc_standard'] = (store.create_entity('Question', identifier=u'vineland_abc_standard', position=13, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'20-160').eid, 20, 160)
    text = u'Vineland Adaptive Behavior Scales Informant'
    vineland_questions['vineland_informant'] = (store.create_entity('Question', identifier=u'vineland_informant', position=14, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'1 = parent; 2 = self').eid, 1, 2)

    store.flush()

    ###############################################################################
    ### SUBJECTS ##################################################################
    ###############################################################################
    groups = []
    for ind in range(NB_SUBJECTS_GROUPS):
        groups.append(store.create_entity('Group', identifier=u'demo_group_%s' % ind,
                                          name=u'Demo group of subjects %s' % ind).eid)
    subjects = []
    for ind in range(NB_SUBJECTS):
        print ind
        date = random_date('age')
        subject = store.create_entity('Subject', identifier='demo_subject_%s' % ind,
                                      gender=SEX[nr.randint(len(SEX))],
                                      date_of_birth=date,
                                      handedness=HANDEDNESS[nr.randint(len(HANDEDNESS))]).eid
        subjects.append(subject)
        store.relate(subject, 'related_studies', study)
        store.relate(subject, 'related_groups', groups[nr.randint(len(groups))])
        # Scores
        for name, (score, _min, _max) in score_defs.iteritems():
            score_value = store.create_entity('ScoreValue', definition=score,
                                              datetime=random_date(), value=nr.randint(_min, _max)).eid
            store.relate(subject, 'related_infos', score_value)

    store.flush()
    store.commit()


    ###############################################################################
    ### QUESTIONNAIRE RUNS ########################################################
    ###############################################################################
    for subject_eid in subjects:
        print subject_eid, 'questionnaire'
        import_questionnaire(store, subject_eid, center, study,
                             ados_questionnaire_eid, ados_questions, 'ados')
        import_questionnaire(store, subject_eid, center, study,
                             vineland_questionnaire_eid, vineland_questions, 'vineland')

    store.flush()
    store.commit()


    ###############################################################################
    ### GENOMIC MEASURES ##########################################################
    ###############################################################################
    for subject_eid in subjects:
        print subject_eid, 'genomics'
        import_genomic(store, subject_eid, center, study, platform)

    store.flush()
    store.commit()


    ###############################################################################
    ### IMAGES MEASURES ###########################################################
    ###############################################################################
    for subject_eid in subjects:
        print subject_eid, 'images'
        import_anat_images(store, subject_eid, center, study, mri_1, mri_2)
        import_fmri_images(store, subject_eid, center, study, mri_1, mri_2)
        import_constrat_images(store, subject_eid, center, study, mri_1, mri_2)
    store.flush()
    store.commit()



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

from logilab.mtconverter import xml_escape

from cubicweb import tags
from cubicweb.predicates import (none_rset, one_line_rset, is_instance, nonempty_rset,
                                 has_related_entities, match_view, match_user_groups,
                                 anonymous_user)
from cubicweb.web import component
from cubicweb.web.views.boxes import SearchBox, EditBox
from cubicweb.web.views.bookmark import BookmarksBox
from cubicweb.web.views.basecomponents import ApplicationName

from cubes.brainomics import COMMENTED_ENTITIES, WIKI_ENTITIES
from cubes.brainomics.views.download import ALL_DOWNLOADABLE

LINKED_ENTITIES = ('Subject',)


################################################################################
### SEARCHBOX ##################################################################
################################################################################
class BrainomicsSearchBox(SearchBox):
    """display a box with a simple search form
    """
    context = _('header-center')
    formdef = (u'<div><form action="%(action)s" class="navbar-search pull-left">'
               u'<input id="norql" type="text" accesskey="q" tabindex="%(tabindex1)s"'
               u'       title="search text" value="%(value)s" name="rql"'
               u'       class="search-query input-xlarge" placeholder="Search"/>'
               u'       <input type="hidden" name="__fromsearchbox" '
               u'              value="1" />'
               u'       <input type="hidden" name="subvid" '
               u'value="tsearch" />'
               u'<!--input tabindex="%(tabindex2)s" type="submit" id="rqlboxsubmit"'
               u'    class="rqlsubmit" value="" /-->'
               u'</form></div>')

    def render(self, w):
        if self._cw.form.pop('__fromsearchbox', None):
            rql = self._cw.form.get('rql', '')
        else:
            rql = ''
        w(self.formdef % {'action': self._cw.build_url('view'),
                          'tabindex1': self._cw.next_tabindex(),
                          'value': xml_escape(rql),
                          'tabindex2': self._cw.next_tabindex()})


###############################################################################
### NAVIGATION BOXES ##########################################################
###############################################################################
class BrainomicsLinksCtx(component.EntityCtxComponent):
    """ Component used to display related informations and links,
    referenced by the registry 'ctx-related-links'.
    """
    context = 'left'
    __regid__ = 'links-ctx'
    order = 1
    __select__ = component.EntityCtxComponent.__select__ & is_instance(*LINKED_ENTITIES)

    def render(self, w, **kwargs):
        links = list(self._cw.vreg['ctx-related-links'].possible_objects(self._cw, rset=self.cw_rset))
        if not links:
            return
        w(u'<div class="well">')
        w(u'<h4>%s</h4>' % self._cw._('Related entities'))
        w(u'<div class="btn-toolbar">')
        # Render links
        for link in links:
            infos = link.related_infos(self.cw_rset)
            if infos:
                w(u'<div class="btn-group-vertical btn-block">')
                w(u'<div class="btn-group">')
                w(u'<a class="btn btn-primary dropdown-toggle" '
                  u'data-toggle="dropdown" href="#">')
                w(u'%s <span class="caret"></span></a>' % link.get_title())
                w(u'<ul class="dropdown-menu">')
                for ahref, title in infos:
                    w(u'<li><a href="%s">%s</a></li>' % (ahref, title))
                w(u'</ul>')
                w(u'</div>')
                w(u'</div>')
        w(u'</div>')
        w(u'</div>')


class BrainomicsLinksDropdown(component.EntityCtxComponent):
    """ Component used to display related informations and links,
    referenced by the registry 'ctx-links'.
    """
    __abstract__ = True
    __registry__ = 'ctx-related-links'

    def get_title(self):
        return

    def related_infos(self, rset):
        return []


class BrainomicsLinksStudies(BrainomicsLinksDropdown):
    """ Component used to display related studies
    """
    __select__ = BrainomicsLinksDropdown.__select__ & is_instance('Subject')
    __regid__ = 'navtoolbar-studies-ctx'

    def get_title(self):
        return self._cw._('Related studies')

    def related_infos(self, rset):
        studies = set()
        for entity in self.cw_rset.entities():
            for study in entity.related_studies:
                studies.add((study.absolute_url(), study.dc_title()))
        return studies


class BrainomicsLinksGroups(BrainomicsLinksDropdown):
    """ Component used to display related groups
    """
    __select__ = BrainomicsLinksDropdown.__select__ & is_instance('Subject')
    __regid__ = 'navtoolbar-groups-ctx'

    def get_title(self):
        return self._cw._('Related groups')

    def related_infos(self, rset):
        groups = set()
        for entity in self.cw_rset.entities():
            for study in entity.related_groups:
                groups.add((study.absolute_url(), study.dc_title()))
        return groups


class BrainomicsLinksCenters(BrainomicsLinksDropdown):
    """ Component used to display related centers
    """
    __select__ = BrainomicsLinksDropdown.__select__ & is_instance('Subject')
    __regid__ = 'navtoolbar-centers-ctx'

    def get_title(self):
        return self._cw._('Related centers')

    def related_infos(self, rset):
        centers = set()
        for entity in self.cw_rset.entities():
            for study in entity.related_centers:
                centers.add((study.absolute_url(), study.dc_title()))
        return centers


###############################################################################
### ADMIN ACTION/BOX ##########################################################
###############################################################################
class BrainomicsEditBox(EditBox):
    context = 'left'
    order = -10
    __select__ = EditBox.__select__ & match_user_groups('users', 'managers')

    def init_rendering(self):
        pass

    def get_actions(self):
        actions = self._cw.vreg['actions'].possible_actions(self._cw, self.cw_rset)
        all_actions = []
        for category in ('mainactions', 'moreactions', 'action-admin'):
            for action in actions.get(category, ()):
                all_actions.append(action)
        return all_actions

    def render(self, w, **kwargs):
        actions = self.get_actions()
        if actions:
            w(u'<div class="well">')
            w(u'<h4>%s</h4>' % self._cw._('Administration tools'))
            for action in actions:
                if hasattr(action, 'title'):
                    icon = getattr(action, 'icon', 'edit')
                    title = self._cw._(action.title)
                    w(u'<a class="btn btn-primary btn-block" href="%s">' % action.url())
                    w(u'<i class="icon-%s icon-white"></i>%s</a>' % (icon, title))
            w(u'</div>')


###############################################################################
### DOWNLOAD BOX ##############################################################
###############################################################################
class BrainomicsDownloadBox(component.CtxComponent):
    __select__ = component.CtxComponent.__select__ & nonempty_rset() & is_instance(*ALL_DOWNLOADABLE)
    context = 'left'
    order = 1
    __regid__ = 'ctx-download-box'

    def render(self, w, **kwargs):
        possible_actions = self._cw.vreg['actions'].possible_actions(self._cw, self.cw_rset)
        links = list(possible_actions.get('download-links', []))
        if links:
            w(u'<div class="well">')
            w(u'<h4>%s</h4>' % self._cw._('Download tools'))
            for link in links:
                url = self._cw.build_url(rql=self.cw_rset.printable_rql(), vid=link.download_vid)
                _id = link.__regid__
                w(u'<a class="btn btn-primary btn-block download-ctx" id="%s" href="%s">'
                  % (_id, url))
                w(u'<i class="icon-download icon-white"></i>%s</a>' % self._cw._(link.title))
            w(u'</div>')
            # Add on load the rql facet change
            self._cw.add_onload("""$(cw).bind('facets-content-loaded',
            cw.cubes.brainomics.changeDownloadUrls);""")


###############################################################################
### REGISTRATION CALLBACK #####################################################
###############################################################################
BookmarksBox.__select__ = BookmarksBox.__select__ & match_user_groups('users', 'managers')
ApplicationName.context = 'header-left'


def registration_callback(vreg):
    vreg.register_all(globals().values(), __name__, (BrainomicsSearchBox,
                                                     BrainomicsEditBox,
                                                     ))
    vreg.register_and_replace(BrainomicsSearchBox, SearchBox)
    vreg.register_and_replace(BrainomicsEditBox, EditBox)
    # Unregister breadcrumbs
    from cubicweb.web.views.ibreadcrumbs import (BreadCrumbEntityVComponent,
                                                 BreadCrumbLinkToVComponent,
                                                 BreadCrumbAnyRSetVComponent,
                                                 BreadCrumbETypeVComponent)
    vreg.unregister(BreadCrumbEntityVComponent)
    vreg.unregister(BreadCrumbAnyRSetVComponent)
    vreg.unregister(BreadCrumbETypeVComponent)
    vreg.unregister(BreadCrumbLinkToVComponent)
    # Unregister logo for now...
    # Unregister anon status component
    from cubicweb.web.views.basecomponents import ApplLogo, AnonUserStatusLink
    vreg.unregister(ApplLogo)
    vreg.unregister(AnonUserStatusLink)



"""bootstrap implementation of base templates

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
# XXX Backport from squareui

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch
from logilab.mtconverter import xml_escape

from cubicweb.utils import UStringIO
from cubicweb.web.views import basetemplates


HTML5 = u'<!DOCTYPE html>'

basetemplates.TheMainTemplate.doctype = HTML5


###############################################################################
### GLOBAL CALL ###############################################################
###############################################################################
@monkeypatch(basetemplates.TheMainTemplate)
def call(self, view):
    self.set_request_content_type()
    self.template_header(self.content_type, view)
    w = self.w
    w(u'<div class="row-fluid">')
    w(u'<div class="span12" id="pageContent">')
    vtitle = self._cw.form.get('vtitle')
    if vtitle:
        w(u'<div class="vtitle">%s</div>\n' % xml_escape(vtitle))
    # display entity type restriction component
    etypefilter = self._cw.vreg['components'].select_or_none(
        'etypenavigation', self._cw, rset=self.cw_rset)
    if etypefilter and etypefilter.cw_propval('visible'):
        etypefilter.render(w=w)
    nav_html = UStringIO()
    if view and not view.handle_pagination:
        view.paginate(w=nav_html.write)
    w(nav_html.getvalue())
    w(u'<div id="contentmain">\n')
    view.render(w=w)
    w(u'</div>\n') # closes id=contentmain
    w(nav_html.getvalue())
    w(u'</div>\n' # closes id=pageContent
      u'</div>\n') # closes row-fluid
    self.template_footer(view)


###############################################################################
### NAV COLUMN AND TOOLBAR ####################################################
###############################################################################
@monkeypatch(basetemplates.TheMainTemplate)
def nav_toolbar(self, view):
    boxes = list(self._cw.vreg['ctxcomponents'].poss_visible_objects(
        self._cw, rset=self.cw_rset, view=view, context='nav-toolbar'))
    if boxes:
        for box in boxes:
            box.render(w=self.w, view=view)

@monkeypatch(basetemplates.TheMainTemplate)
def nav_column(self, view, context):
    boxes = list(self._cw.vreg['ctxcomponents'].poss_visible_objects(
        self._cw, rset=self.cw_rset, view=view, context=context))
    if boxes:
        getlayout = self._cw.vreg['components'].select
        self.w(u'<div id="aside-main-%s" class="span3">\n' % context)
        self.w(u'<div class="navboxes" id="navColumn%s">\n' % context.capitalize())
        for box in boxes:
            box.render(w=self.w, view=view)
        self.w(u'</div></div>')
    return len(boxes)


###############################################################################
### HEADER ####################################################################
###############################################################################
@monkeypatch(basetemplates.TheMainTemplate)
def template_html_header(self, content_type, page_title,
                         additional_headers=()):
    w = self.whead
    lang = self._cw.lang
    self.write_doctype()
    # explicitly close the <base> tag to avoid IE 6 bugs while browsing the DOM
    self._cw.html_headers.define_var('BASE_URL', self._cw.base_url())
    self._cw.html_headers.define_var('DATA_URL', self._cw.datadir_url)
    w(u'<meta http-equiv="content-type" content="%s; charset=%s"/>\n'
      % (content_type, self._cw.encoding))
    w(u'<meta name="viewport" content="initial-scale=1.0; '
      u'maximum-scale=1.0; width=device-width; "/>')
    w(u'\n'.join(additional_headers) + u'\n')
    # FIXME this is a quick option to make cw work in IE9
    # you'll lose all IE9 functionality, the browser will act as IE8.
    w(u'<meta http-equiv="X-UA-Compatible" content="IE=8" />\n')
    w(u'<!-- Le HTML5 shim, for IE6-8 support of HTML elements -->\n'
      u'  <!--[if lt IE 9]>\n'
      u'        <script src="%s"></script>\n'
      u'  <![endif]-->\n' % self._cw.data_url('js/html5.js'))
    self.wview('htmlheader', rset=self.cw_rset)
    if page_title:
        w(u'<title>%s</title>\n' % xml_escape(page_title))

@monkeypatch(basetemplates.TheMainTemplate)
def template_body_header(self, view):
    w = self.w
    w(u'<body>\n')
    self.wview('header', rset=self.cw_rset, view=view)
    # Toolbar
    w(u'<div id="toolbar" class="container-fluid">\n'
      u'<div class="row-fluid offset3">\n')
    self.nav_toolbar(view)
    w(u'</div></div>\n')
    # Page
    w(u'<div id="page" class="container-fluid">\n'
      u'<div class="row-fluid">\n')
    #w(u'<div class="span3">')
    nb_boxes = self.nav_column(view, 'left')
    #w(u'</div>')
    if nb_boxes is not None and nb_boxes:
        content_span = 9
    else:
        content_span = 12
    w(u'<div id="contentColumn" class="span%s">' % content_span)
    components = self._cw.vreg['components']
    rqlcomp = components.select_or_none('rqlinput', self._cw, rset=self.cw_rset)
    if rqlcomp:
        rqlcomp.render(w=self.w, view=view)
    msgcomp = components.select_or_none('applmessages', self._cw, rset=self.cw_rset)
    if msgcomp:
        msgcomp.render(w=self.w)
    self.content_header(view)

@monkeypatch(basetemplates.HTMLPageHeader)
def main_header(self, view):
    """build the top menu with authentification info and the rql box"""
    spans = {'headtext': 3,
             'header-center': 7,
             'header-right': 2,
             }
    w = self.w
    w(u'<div id="header" class="navbar">'
      u'<div class="navbar-inner">'
      u'<div class="container">'
      u'<div class="row-fluid">')
    prev_span_size = 0
    for colid, context in self.headers:
        components = self._cw.vreg['ctxcomponents'].poss_visible_objects(
            self._cw, rset=self.cw_rset, view=view, context=context)
        if components:
            span_size = spans.get(colid, 2) + prev_span_size
            prev_span_size = 0
            w(u'<div id="%s" class="span%s">' % (colid, span_size))
            klass = ' pull-right' if context in ('header-center', 'header-right') else ''
            w(u'<ul class="nav%s">' % klass)
            for comp in components:
                w(u'<li>')
                comp.render(w=w)
                w(u'</li>')
            w(u'</ul>')
            w(u'</div>')
        else:
            # Keep size to make a bigger span the next time
            prev_span_size += spans.get(colid, 2)
    w(u'</div></div></div></div>\n')
    # get login form to display it as modal window / Backport from Orbui
    login = self._cw.vreg['forms'].select_or_none('logform', self._cw)
    if login:
        self.w(u'<div id="loginModal" class="modal hide fade in">'
               u'<div class="modal-header">'
               u'<a class="close" data-dismiss="modal">x</a>'
               u'<h3>%s</h3>'
               u'</div>'
               u'<div class="modal-body">' % self._cw._('log in'))
        login.render(w=self.w)
    self.w(u'</div>'
           u' <div class="modal-footer"></div>'
           u'</div>')



###############################################################################
### FOOTER ####################################################################
###############################################################################
@monkeypatch(basetemplates.TheMainTemplate)
def template_footer(self, view=None):
    self.w(u'<div class="row">')
    self.content_footer(view)
    self.w(u'</div>')
    self.w(u'</div>\n') # XXX closes div#contentColumn span9 in template_body_header
    self.nav_column(view, 'right')
    self.w(u'</div>\n') # XXX closes div#page in template_body_header
    self.wview('footer', rset=self.cw_rset)
    self.w(u'</div>'    # closes class="row-fluid"
           u'</div>')   # closes class="container-fluid"
    self.w(u'</body>')


@monkeypatch(basetemplates.HTMLPageFooter)
def call(self, **kwargs):
    self.w(u'<footer id="pagefooter">')
    self.w(u'<div id="footer" class"container">')
    self.footer_content()
    self.w(u'</div>')
    self.w(u'</footer>')



import os
import os.path as osp
import tempfile
import logging
import csv
from cubicweb.utils import make_uid
from cubicweb.devtools.testlib import CubicWebTC
from cubes.brainomics.ccplugin import copy_file, update_ident_fpath, get_ids_uids_from_csv



class TestBrainomicsRemakeUid(CubicWebTC):

    def setup_database(self):
        super(TestBrainomicsRemakeUid, self).setup_database()
        ce = self.request().create_entity
        self.rql = self.session.execute
        _study = ce('Study', name=u'Test study', data_filepath=u'/tmp',
                    description=u'Test study with dummy data', keywords=u'test;dummy')
        self.subject_1 = ce('Subject', identifier=u'demo_subject_1', gender=u'female',
                            handedness=u'right')
        self.subject_2 = ce('Subject', identifier=u'demo_subject_2', gender=u'male',
                            handedness=u'ambidextrous')
        self.scan_11 = ce('Scan', identifier=u'anat_1709',
                          label=u'anatomy', type=u'normalized T1', format=u'nii.hz',
                          completed=True, valid=True, filepath=u'1709/anat.nii.gz',
                          concerns=self.subject_1, related_study=_study)
        self.scan_12 = ce('Scan', identifier=u'anat_1899',
                          label=u'anatomy', type=u'normalized T1', format=u'nii.hz',
                          completed=True, valid=True, filepath=u'1899/anat.nii.gz',
                          concerns=self.subject_1, related_study=_study)
        self.scan_21 = ce('Scan', identifier=u'anat_2325',
                          label=u'anatomy', type=u'normalized T1', format=u'nii.hz',
                          completed=True, valid=True, filepath=u'2325/anat.nii.gz',
                          concerns=self.subject_2, related_study=_study)
        self.scan_22 = ce('Scan', identifier=u'anat_3785',
                          label=u'anatomy', type=u'normalized T1', format=u'nii.hz',
                          completed=True, valid=True, filepath=u'3785/anat.nii.gz',
                          concerns=self.subject_2, related_study=_study)
        _center = ce('Center', identifier=u'testing_center', name=u'Testing Center')
        self.assessment = ce('Assessment', identifier=u'assessment_3125',
                             related_study=_study, reverse_holds=_center,
                             reverse_concerned_by=self.subject_1)
        self.log = logging.getLogger('brainomics test: remake-uuid')
        fh = logging.FileHandler('brainomics_test.log')
        self.log.addHandler(fh)

    def test_copy_file(self):
        for fpath, full_fpath in ((scan.filepath, scan.full_filepath)
                                  for scan in (self.scan_11, self.scan_12, self.scan_21, self.scan_22)):
            dest_path = osp.join(full_fpath.split(fpath)[0], 
                                 unicode(make_uid()) + '_' + osp.basename(fpath))
            with tempfile.NamedTemporaryFile(suffix=osp.basename(fpath)) as f:
                copy_file(f.name, dest_path, logger=self.log)
            if osp.isfile(dest_path):
                os.remove(dest_path)

    def test_update_fpath_nodelete(self):
        fpath_query = 'Any F WHERE X is IN (Scan, GenomicMeasure), X filepath F'
        old_fpaths = self.rql(fpath_query, build_descr=False)
        for s in self.rql('Any X WHERE X is Subject'):
            update_ident_fpath(self.session, s[0], unicode(make_uid()), logger=self.log)
        new_fpaths = self.rql(fpath_query, build_descr=False)
        self.assertNotEqual(old_fpaths, new_fpaths)

    def test_update_fpath_delete(self):
        fpath_query = 'Any F WHERE X is IN (Scan, GenomicMeasure), X filepath F'
        old_fpaths = self.rql(fpath_query, build_descr=False)
        for s in self.rql('Any X WHERE X is Subject'):
            update_ident_fpath(self.session, s[0], unicode(make_uid()), delete=True, logger=self.log)
        new_fpaths = self.rql(fpath_query, build_descr=False)
        self.assertNotEqual(old_fpaths, new_fpaths)

    def test_update_ident(self):
        ident_query = 'Any I WHERE X is IN (Scan, GenomicMeasure, QuestionnaireRun), X identifier I'
        old_idents = self.rql(ident_query, build_descr=False)
        for s in self.rql('Any X WHERE X is Subject'):
            update_ident_fpath(self.session, s[0], unicode(make_uid()), logger=self.log)
        new_idents = self.rql(ident_query)
        self.assertNotEqual(old_idents, new_idents)

    def test_update_ident_assessment(self):
        assess_ident_query = 'Any I WHERE X is Assessment, X identifier I'
        old_idents = self.rql(assess_ident_query, build_descr=False)
        for s in self.rql('Any X WHERE X is Subject'):
            update_ident_fpath(self.session, s[0], unicode(make_uid()), logger=self.log)
        new_idents = self.rql(assess_ident_query, build_descr=False)
        self.assertNotEqual(old_idents, new_idents)

    def test_get_uid_from_csv(self):
        old_new_uuids = {}
        identifiers_rset = self.rql('Any I WHERE X is Subject, X identifier I', 
                                    build_descr=False)
        # Generate test CSV
        test_csv_fname = 'external_uuids.csv'
        with open(test_csv_fname, 'wb') as csvf:
            csvw = csv.writer(csvf, delimiter='\t')
            for old_identifier in identifiers_rset: 
                old_new_uuids[old_identifier[0]] = u'test_' + unicode(make_uid())
                csvw.writerow([old_identifier[0], old_new_uuids[old_identifier[0]]])
        # Verify that the new UUIDs are correctly retrieved from the CSV
        ids_uids = get_ids_uids_from_csv(test_csv_fname, logger=self.log)
        for old_identifier in identifiers_rset:
            self.assertEqual(old_new_uuids[old_identifier[0]], 
                             ids_uids[old_identifier[0]])




if __name__ == '__main__':
    from logilab.common.testlib import unittest_main
    unittest_main()



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

""" Registration and security tools
"""
from logilab.common.decorators import monkeypatch

from yams.schema import role_name

from cubicweb.server import repository
from cubicweb.server.ssplanner import EditedEntity
from cubicweb import ValidationError


###############################################################################
### REGISTRATION ##############################################################
###############################################################################
@monkeypatch(repository.Repository)
def register_user(self, login, password, email=None, **kwargs):
    with self.internal_session() as session:
        # for consistency, keep same error as unique check hook (although not required)
        errmsg = session._('the value "%s" is already used, use another one')
        if (session.execute('CWUser X WHERE X login %(login)s', {'login': login},
                            build_descr=False)
            or session.execute('CWUser X WHERE X use_email C, C address %(login)s',
                               {'login': login}, build_descr=False)):
            qname = role_name('login', 'subject')
            raise ValidationError(None, {qname: errmsg % login})
        # we have to create the user
        user = self.vreg['etypes'].etype_class('CWUser')(session)
        if isinstance(password, unicode):
            # password should *always* be utf8 encoded
            password = password.encode('UTF8')
        kwargs['login'] = login
        kwargs['upassword'] = password
        self.glob_add_entity(session, EditedEntity(user, **kwargs))
        ###################################
        # Monkeypatch register user to create the user in the "guests" group
        # for security reason (just read ability)
        session.execute('SET X in_group G WHERE X eid %(x)s, G name "guests"',
                        {'x': user.eid})
        ###################################
        if email or '@' in login:
            d = {'login': login, 'email': email or login}
            if session.execute('EmailAddress X WHERE X address %(email)s', d,
                               build_descr=False):
                qname = role_name('address', 'subject')
                raise ValidationError(None, {qname: errmsg % d['email']})
            session.execute('INSERT EmailAddress X: X address %(email)s, '
                            'U primary_email X, U use_email X '
                            'WHERE U login %(login)s', d, build_descr=False)
        session.commit()
    return True



# -*- coding: utf-8 -*-
# copyright 2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

import sys
import os.path as osp
import glob

import nibabel as nb

from cubicweb.dataimport import SQLGenObjectStore as Store

from cubes.brainomics.importers.helpers import get_image_info


if __name__ == '__main__':
    base_path = osp.abspath(sys.argv[4])
    store = Store(session)
    global_study = store.create_entity('Study', data_filepath=unicode(base_path), name = u'mindboggle',
                                       description=u'''The Mindboggle-101 dataset includes manually labeled anatomical regions for 101 healthy subjects. The manually edited cortical labels follow sulcus landmarks according to the Desikan-Killiany-Tourville (DKT) protocol. See http://www.frontiersin.org/Brain_Imaging_Methods/10.3389/fnins.2012.00171/full “101 labeled brain images and a consistent human cortical labeling protocol”
Arno Klein, Jason Tourville. Frontiers in Brain Imaging Methods. 6:171. DOI: 10.3389/fnins.2012.00171
    and http://mindboggle.info/data.html''')
    for study_path in ('Extra-18_volumes', 'MMRR-21_volumes', 'NKI-RS-22_volume',
                       'NKI-TRT-20_volumes', 'OASIS-TRT-20_volumes'):
        full_study_path = osp.join(base_path, study_path)
        if not osp.exists(full_study_path):
            print '%s does not exist, skip it' % full_study_path
            continue
        # Create study
        count_subject = 0
        study = store.create_entity('Study', data_filepath=unicode(full_study_path),
                                    name = unicode(study_path))
        for subject_path in glob.iglob(osp.join(full_study_path,'*')):
            subject_id = subject_path.split('/')[-1]
            print '--->', subject_id
            subject = store.create_entity('Subject', identifier=unicode(subject_id),
                                          gender=u'unknown', handedness=u'unknown')
            store.relate(subject.eid, 'related_studies', study.eid)
            store.relate(subject.eid, 'related_studies', global_study.eid)
            assessment = store.create_entity('Assessment',identifier=u'label',
                                             related_study=study.eid,
                                             protocol=u'mindboggle-label')
            store.relate(subject.eid, 'concerned_by', assessment.eid)
            for image in glob.iglob(osp.join(full_study_path, subject_path, '*.nii.gz')):
                image_id = image.split('/')[-1].split('.')[0]
                anat = {'identifier': u'%s_%s' % (subject_id, image_id),
                        'label': u'labels' if 'labels' in image_id else u'anatomy',
                        'format': u'nii.gz',
                        'type': u'normalized T1',
                        'related_study': study.eid,
                        'filepath': unicode(osp.relpath(image, start=full_study_path)),
                        'valid': True, 'completed': True, 'description': 'Image from Mindboggle'}
                mri_data = store.create_entity('MRIData', sequence=u'T1')
                # MRI data
                mri_data = {'sequence': u'T1'}
                mri_data.update(get_image_info(image))
                mri_data = store.create_entity('MRIData', **mri_data)
                anat['has_data'] = mri_data.eid
                anat['related_study'] = study.eid
                anat = store.create_entity('Scan', **anat)
                store.relate(anat.eid, 'concerns', subject.eid, subjtype='Scan')
                store.relate(assessment.eid, 'generates', anat.eid, subjtype='Assessment')
            count_subject += 1
        # Flush
        store.flush()
        store.commit()
        print '-------> %s subjects imported for %s' % (count_subject, study_path)
